#include <qfont.h>
#include <qpopupmenu.h>
#include <qfiledialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include "channel.h"
#include "icons.h"

Channel::Channel(QWidget *parent, const QString &name, QColor bcolor, QFont chatfont, bool prv, bool brd): QTextBrowser(parent), activity(false), filename(QString::null)
{
    this->prv = prv;
    board = brd;
    chname = name;
    setReadOnly(true);
//    setFocusPolicy(QWidget::NoFocus);
    setTextFormat(Qt::RichText);
    setPaletteBackgroundColor(bcolor);
    setLinkUnderline(false);
    setWrapPolicy(QTextEdit::AtWordOrDocumentBoundary);
    setFont(chatfont);
    setTopic("");
    setMouseTracking (true);
    setupActions();
}

Channel::~Channel()
{
}

void Channel::setupActions()
{
    selectAllAction = new QAction(this, "SelectAllAction");
    selectAllAction->setAccel(CTRL+Key_A);    
    connect(selectAllAction, SIGNAL(activated()), this, SLOT(selectAll()));    
    if (!board)
    {
	saveAction = new QAction(this, "SaveAction");
	saveAction->setAccel(CTRL+Key_S);    
	connect(saveAction, SIGNAL(activated()), this, SLOT(save()));

	leaveAction = new QAction(this, "LeaveAction");
	leaveAction->setAccel(CTRL+Key_Q);
	connect(leaveAction, SIGNAL(activated()), this, SLOT(leave()));
    }
}

void Channel::leave()
{
    emit leave(chname);
}

void Channel::keyPressEvent ( QKeyEvent * e )
{
	if ((e->key() != Qt::Key_Control) && (e->state() != Qt::ControlButton)) 
	{
		if (e->key() != Qt::Key_Return && e->key() != Qt::Key_Enter) emit keypressed(e->text());
		else 
			emit keypressed("");
		return;
	}
	QTextBrowser::keyPressEvent(e);
}

QPopupMenu* Channel::createPopupMenu(const QPoint &pos)
{
    QPopupMenu *menu = new QPopupMenu(this);
    menu->insertItem(Icons::icon_save, tr("Save"), this, SLOT(save()), CTRL+Key_S);
    menu->insertItem(Icons::icon_saveas, tr("Save As"), this, SLOT(saveAs()));
    menu->insertSeparator();
    menu->insertItem(tr("Select all"), this, SLOT(selectAll()), CTRL+Key_A);
    menu->insertItem(Icons::icon_copy, tr("Copy"), this, SLOT(copy()), CTRL+Key_C);
    menu->insertSeparator();
    if (!board)
    {
	if (!prv)
	{
	    menu->insertItem(tr("Clear topic"), this, SLOT(topic_clear()));
	    menu->insertSeparator();
	}
	menu->insertItem(Icons::icon_leave, tr("Leave"), this, SLOT(leave()), CTRL+Key_Q);
    }
    return menu;
}

bool Channel::doSave()
{
    QFile f(filename);
    if (!f.open(IO_WriteOnly))
	return false;
    QTextStream str(&f);
    str << text();
    f.close();
    return true;
}

void Channel::save()
{
    if (filename.isNull())
	saveAs();
    doSave();
}

void Channel::saveAs()
{
    QString s = QFileDialog::getSaveFileName(QString::null, 
	tr("HTML files (*.htm)"), 
	this, 
	"open file dialog",
	tr("Save as..."));
    if (!s.isNull())
        filename = s;
    doSave();
}

void Channel::topic_clear()
{
    emit sig_topic_clear(chname);
}

void Channel::mouseMoveEvent ( QMouseEvent * e )
{
    emit activityDetected();
}
