#include "chatline.h"
#include "global.h"
#include "icons.h"
#include <qpopupmenu.h>

//#include <iostream>

ChatLine::ChatLine(QWidget *parent, const char *name): QTextEdit(parent, name), historypos(1)
{
    history.append("");
    maxhistory = 100;
    setTextFormat(Qt::PlainText);
    setWordWrap(QTextEdit::NoWrap);
    setHScrollBarMode(QScrollView::AlwaysOff);
    setVScrollBarMode(QScrollView::AlwaysOff);
}

ChatLine::~ChatLine()
{
}

QPopupMenu * ChatLine::createPopupMenu(const QPoint &pos)
{
    menu = new QPopupMenu(this);
    smilemenu = new QPopupMenu(this);
    for (int i = 1; i <= SM_SLEEP; i++) 
	smilemenu->insertItem(QIconSet(QPixmap::fromMimeSource(smilespack[i - 1].file)), QString(smilespack[i - 1].smile).replace("&", "&&"), i);
 
    menu->insertItem(Icons::icon_cut, tr("Cut"), ID_CUT);
    menu->insertItem(Icons::icon_copy, tr("Copy"), ID_COPY);
    menu->insertItem(Icons::icon_paste, tr("Paste"), ID_PASTE);
    menu->insertSeparator();
    menu->insertItem(Icons::icon_smile, tr("Smiles"), smilemenu);
    connect(smilemenu, SIGNAL(activated(int)), this, SLOT(smileMenuAction(int)));
    connect(menu, SIGNAL(activated(int)), this, SLOT(standardMenuAction(int)));
    
    return menu;
}

void ChatLine::smileMenuAction(int opt)
{
    QString tmp(smilespack[opt - 1].smile);
    insert(tmp);
}

void ChatLine::standardMenuAction(int opt)
{
    switch(opt)
    {
    case ID_CUT:
	cut();
	break;
    case ID_COPY:
	copy();
	break;
    case ID_PASTE:
	paste();
	break;
    default: break;
    }
}

void ChatLine::toggleMultiLine()
{
    setMultiline(!mline);
}

void ChatLine::setMultiline(bool multi)
{
    if (multi)
	{
		 this->setFixedHeight((this->fontMetrics().lineSpacing() + 7) * 4); //Four lines in multiline mode
		 setWordWrap(QTextEdit::WidgetWidth);
		 setHScrollBarMode(QScrollView::Auto);
		 setVScrollBarMode(QScrollView::Auto);
	 }
    else 
	 {
		 this->setFixedHeight(this->fontMetrics().lineSpacing() + 7);
		 setWordWrap(QTextEdit::NoWrap);
		 setHScrollBarMode(QScrollView::AlwaysOff);
		 setVScrollBarMode(QScrollView::AlwaysOff);
	 }
	 mline = multi;
}

void ChatLine::sayNick(const QString &nick)
{
	QString tmp, to;
    
	tmp = text();

	to = nick + ": ";
    
	if (tmp.left(to.length()) == to) to = "";
    
	setText(to + tmp);
	moveCursor(QTextEdit::MoveEnd, false);
}

QString ChatLine::convertMsg(const QString &txt)
{
	QString str = txt;
	QString tmp = QString("`qwertyuiop[]asdfghjkl;'zxcvbnm,./~QWERTYUIOP{}ASDFGHJKL:\"ZXCVBNM<>?");
	QString tmp2 = QString(tr("`qwertyuiop[]asdfghjkl;'zxcvbnm,./~QWERTYUIOP{}ASDFGHJKL:\"ZXCVBNM<>?"));
	for(uint i = 0; i < txt.length(); i++)
	{
		for(uint t = 0; t < tmp.length(); t++) 
		{
			if(str.at(i) == tmp.at(t)) 
			{
				str[i] = tmp2.at(t);
				break;
			}
			if(str.at(i) == tmp2.at(t))
			{
				str[i] = tmp.at(t);
				break;
			}
		}
	}
	return str;
}

void ChatLine::keyPressEvent(QKeyEvent *e)
{
    if ((e->key() == Qt::Key_Backspace) && (e->state() == Qt::ControlButton))
    {
	int para, index;
	getCursorPosition (&para, &index);
	QString txt = this->selectedText();
	if (!txt.isEmpty()) 
	    this->insert(convertMsg(txt));
	else 
	    if (this->text().contains(':'))
	    {
		this->setSelection(0, this->text().find(':') + 1, 0, length());
		this->insert(convertMsg(selectedText()));
	    }
	    else
		this->setText(convertMsg(this->text()));
	setCursorPosition(para, index);
	return e->accept();
    }
    if (e->key() == Qt::Key_Up)
    {
	if (!mline || (mline && e->state() == Qt::ControlButton))
	{
	    if (history[historypos] != text()) addToHistory();
	    if (--historypos < 0)
		historypos = history.count() - 1;
	    
	    setText(history[historypos]);
	    moveCursor(QTextEdit::MoveEnd, false);
	    return e->accept();
	}
    }
    if (e->key() == Qt::Key_Down)
    {
	if (!mline || (mline && e->state() == Qt::ControlButton))
	{
	    if (history[historypos] != text()) addToHistory();
	    
	    if ((uint)++historypos >= history.count())
		historypos = 0;
	    
	    setText(history[historypos]);
	    moveCursor(QTextEdit::MoveEnd, false);
	    return e->accept();
	}
    }
    if (e->key() == Qt::Key_Return || e->key() == Qt::Key_Enter)
    {
	if (!mline && length() == 0) return e->accept();
	if (!mline || (mline && e->state() == Qt::ControlButton))
	{
	    addToHistory();
	    emit sendLine();
	    return e->accept();
	}
    }
    QTextEdit::keyPressEvent(e);    
}

void ChatLine::addToHistory()
{
    if (text().isEmpty()) return;
    
    history.push_back(text());
    historypos = history.count();

    if (history.count() >= maxhistory)
	history.pop_front();
}

QStringList ChatLine::getHistory() const
{
    return history;
}
