#ifndef __CHATTAB_H
#define __CHATTAB_H

#include "channel.h"
#include <qtabwidget.h>
#include <qmap.h>

class QAction;

class ChatTab: public QTabWidget
{
    Q_OBJECT
    
	private:
	    typedef QMap<QString, Channel *> ChannelMap;
	    
	    ChannelMap cmap;
	    Channel *prevdel;
	    Channel *board;
	    
	    QString lastactive;
	    
	    bool del(const QString &name);
    
	public slots:
	    void clearCurrent();
	    void scrollDown(const QString &chn);
	    void scrollDownCurrent();
	    
	private slots:
	    void tabChanged(QWidget *widget);
	    void onChannelDoubleClicked(int para, int pos);
	    void on_linkClicked(const QString &link);
	    
	signals:
	    void leave(const QString &channel);
	    void keypressed(const QString & s);
	    void sig_topic_clear(const QString &);
	    void sig_linkClicked(const QString &link, const QString &chn);	
	    void saveChatLineText(const QString &chn);
	    void loadChatLineText(const QString &chn);
	    void boardActivated();
	    void boardDisactivated();
	    void updateCaption();
	    void actDetected();
	    void channelDoubleClicked(const QString &chn, int para, int pos);
	public:
	    ChatTab(QWidget *parent, const char * name);
	    
	    ~ChatTab();
	    
	    bool isPrivate(const QString &name);
	    bool isPrivate() { return isPrivate(current()); }
	    bool isBoard(const QString &name);
	    bool addChannel(const QString &name, QColor back, QFont chatfont);
	    bool addBoard(QFont chatfont);
	    bool delChannel(const QString &name);
	    bool addPrivate(const QString &name, QColor bcolor, QFont chatfont);
	    bool renamePrivate(const QString &from, const QString &to);
	    bool delPrivate(const QString &name);
	    bool delBoard();
	    bool clearTopic(const QString &name);
	    bool setTopic(const QString &name, const QString &topic);
	    bool write(const QString &channel, const QString &str, QColor color);
	    bool writeToBoard(const QString &str);
	    bool exists(const QString &name, bool cs = false);
	    bool clearPage(const QString &name);
	    bool clearBoard();
	    QString getTopic(const QString &name);
	    QString current() { return currentPage()->name(); }
	    QString channelsList();
	    QStringList privatesList();
	    bool saveChatLine(const QString &chn, const QString &text);
	    QString loadChatLine(const QString &chn);
	    
	    Channel *currentPage()
	    { 
		return static_cast<Channel *>(QTabWidget::currentPage());
	    }

	};

#endif

