/***************************************************************************
                          historyform.ui.h - description
                             -------------------
    begin                : Thu Nov 10 18:08:07 EST 2005
    copyright         : (C) 2004-2007 by Sergey Litvinov
    email                : linch@adsl.by
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify                                             *
 *   it under the terms of the GNU General Public License as published by                                          *
 *   the Free Software Foundation; either version 2 of the License, or 		                         *
 *   (at your option) any later version.                                                                                                          *
 ***************************************************************************/

void HistoryForm::init()
{
    historyBrowser->setLinkUnderline(false);
}

void HistoryForm::setup(const QString& p)
{
	path = p;
	header();
	updateLog();
	showMain();
	footer("#G", "#G");
}

bool HistoryForm::updateLog()
{
    // получение информации из лог-файла и распределение ее по личным, сообщениям и каналам
    char ttype;
    QString tdate, ttime, tsrc, tuser, tmessage;
	 
	 chn.clear();
	 msg.clear();
	 prv.clear();
	 m.clear();
	
    QFile file(path);
    if (file.open(IO_ReadOnly)) 
    {
	buf = (char *)malloc(file.size());
	file.readBlock(buf, file.size());
	QString bigstr = QString::fromUtf8(buf, file.size());
	file.close();
	free(buf);
	QStringList biglist = QStringList::split(SEPARATOR, bigstr);
	if (biglist.isEmpty()) return false;
	for(uint i = 0; i <biglist.count(); i++)
	{
	    tdate = "";
	    ttime = "";
	    tsrc = "";
	    ttype = '\0';
	    tuser = "";
	    tmessage = "";
	    QStringList datelist = QStringList::split("<DATE>", biglist[i]);
	    if (datelist.count() == 2)
	    {
		tdate = datelist[0];
		QStringList timelist = QStringList::split("<TIME>", datelist[1]);
		if(timelist.count() == 2)
		{
		    ttime = timelist[0];
		    QStringList typelist = QStringList::split("<CHN>", timelist[1]);
		    ttype = CHN_TYPE;
		    if (typelist.count() != 2)
		    {
			typelist = QStringList::split("<MSG>", timelist[1]);
			ttype = MSG_TYPE;
			if (typelist.count() != 2) 
			{
			    typelist = QStringList::split("<PRV>", timelist[1]);
			    ttype = PRV_TYPE;  
			}
		    }
		    if (typelist.count() == 2) 
		    {
			tsrc = typelist[0];
			QStringList userlist = QStringList::split("<USR>", typelist[1]);
			if(userlist.count() == 2)
			{
			    tuser = userlist[0];
			    tmessage = userlist[1];
			}
		    }
		}
	    }
	    m.append(LogMessage(tdate, ttime, tsrc, ttype, tuser, tmessage));
	}
	
	LogMessageList::iterator it;
	for (it = m.begin(); it != m.end(); it++)
	{
	    switch((*it).type())
	    {
			case CHN_TYPE:
			    if(!chn.contains((*it).src())) chn.append((*it).src());
			    break;
			case MSG_TYPE:
			    if(!msg.contains((*it).src())) msg.append((*it).src());
			    break;
			case PRV_TYPE:
			    if(!prv.contains((*it).src())) prv.append((*it).src());
			    break;
			default: break;
			}		
	    
	}
    }
    return true;
}

void HistoryForm::header()
{
	historyBrowser->setText(tr("<p align=\"center\"><font color=\"#00007f\" size=\"+4\">TriX history</font></p>"));
}

void HistoryForm::footer(const QString &back, const QString &refresh)
{
	historyBrowser->append(QString(tr("<table width=\"100%\"><tr><td align=\"center\"><a href=\"%1\">Back</a></td><td align=\"center\"><a href=\"#G\">Menu</a></td><td align=\"center\"><a href=\"#R%2\">Refresh</a></td><td align=\"center\"><a href=\"#Q\">Exit</a></td></tr></table>")) .arg(back).arg(refresh));
}

void HistoryForm::showMain()
{
    QString thb;
	
    thb.append(tr("<table width=\"100%\"><tr><td><table width=\"100%\"><tr><td align=\"center\">Channels</td></tr>"));
    for(uint i = 0; i < chn.count(); i++)
	thb.append(QString("<tr><td align=\"center\"><a href=\"#FH%1\">%2</a></td></tr>")
		   .arg(chn[i])
		   .arg(chn[i]));
    thb.append(tr("</table></td><td><table width=\"100%\"><tr><td align=\"center\">Messages</td></tr>"));
    for(uint i = 0; i < msg.count(); i++)
	thb.append(QString("<tr><td align=\"center\"><a href=\"#FM%1\">%2</a></td></tr>")
		   .arg(msg[i])
		   .arg(msg[i]));
    thb.append(tr("</table></td><td><table width=\"100%\"><tr><td align=\"center\">Privates</td></tr>"));
    for(uint i = 0; i < prv.count(); i++)
	thb.append(QString("<tr><td align=\"center\"><a href=\"#FP%1\">%2</a></td></tr>")
		   .arg(prv[i])
		   .arg(prv[i]));
    thb.append("</table></td></tr></table>");
    historyBrowser->append(thb);
}

void HistoryForm::request(const QString &req)
{
    on_linkClicked(req);
}

void HistoryForm::on_linkClicked(const QString &str)
{
    int cmd, subcmd;
    QString link, date, src;
    
    if (str[0] == '#') 
    {
	cmd = str[1];
	    
	link = str.right(str.length() - 2);
		
	switch(cmd)
	{
	case 'Q':
		close();
		break;
	case 'R':
		updateLog();
		on_linkClicked(link);
		break;
	case 'G':
		historyBrowser->clear();
		header();
		showMain();
		footer("#G", "#G");
		break;
	    
	case 'F':
	    subcmd = link[0].latin1();
	    link = link.right(link.length() - 1);
	    
	    historyBrowser->clear();
	    header();
	    
	    switch(subcmd)
	    {
	    case CHN_TYPE:
		historyBrowser->append(tr("<p align=\"center\">Channel: %1</p>") .arg(link));	
		break;
	    case MSG_TYPE:
		historyBrowser->append(tr("<p align=\"center\">Messages from(to): %1</p>") .arg(link));
		break;
	    case PRV_TYPE:
		historyBrowser->append(tr("<p align=\"center\">Privates with: %1</p>") .arg(link));	
		break;
		
	    default: break;
	    }
		 listDate(link, subcmd);
		 footer("#G", str);
	    break;
	case 'A':
	    subcmd = link[0].latin1();
	    link = link.right(link.length() - 1);
	    historyBrowser->clear();
	    header();
	    historyBrowser->append(tr("<p align=\"center\">All messages for: %1</p>") .arg(link));
	    listAll(link, subcmd);
	    footer(QString("#F") + QChar(subcmd) + link, str);
	    break;
	
	case 'D':
	    int pos;
	    subcmd = link[0].latin1();
	    link = link.right(link.length() - 1);
	    pos = link.find('?');
	    date = link.left(pos);
	    src = link.mid(pos + 1);
	    historyBrowser->clear();
	    header();
	    historyBrowser->append(tr("<p align=\"center\">Messages for: %1</p>") .arg(src));
	    historyBrowser->append(tr("<p align=\"center\"><font color=BLUE>Date: %1</font></p>") .arg(date));
	    listByDate(date, subcmd, src);
		 footer(QString("#F") + QChar(subcmd) + src, str);
	    break;
	    
	default: break;
	}
    }
}

void HistoryForm::listDate(const QString & link, int cmd)
{
    QString thb;
    QStringList dt;

    thb.append(QString(tr("<table><tr><td align=\"center\"><a href=\"#A%1%2\">All</a></td></tr>")) .arg(QChar(cmd)) .arg(link));
			
    LogMessageList::iterator it;
    for(it = m.begin(); it != m.end(); it++)
	if(((*it).type() == cmd) && ((*it).src() == link) && (!dt.contains((*it).date()))) {
	dt.append((*it).date());
	thb.append(QString("<tr><td align=\"center\"><a href=\"#D%1%2?%3\">%4</a></td></tr>") .arg(QChar(cmd)) .arg((*it).date()) .arg(link) .arg((*it).date()));
    }
			
    thb.append("</table>");
    historyBrowser->append(thb);
}

void HistoryForm::listAll(const QString & link, int cmd)
{
    QStringList dt;
	
    LogMessageList::iterator it;
    for(it = m.begin(); it != m.end(); it++) 
    {
	if(((*it).type() == cmd) && ((*it).src() == link)) 
	{
	    if (!dt.contains((*it).date())) 
	    {
		dt.append((*it).date());
		historyBrowser->append(tr("<p align=\"center\"><font color=BLUE>Messages for: %1</font></p>") .arg((*it).date()));
	    }
	    historyBrowser->append(QString("%1%2: %3") .arg((*it).time()) .arg((*it).user()) .arg((*it).message()));
	}
    }
}

void HistoryForm::listByDate(const QString & date, int cmd, const QString &src)
{
    QString thb;
    QStringList dt;
	
    LogMessageList::iterator it;
    for(it = m.begin(); it != m.end(); it++)
	if(((*it).date() == date) && ((*it).type() == cmd) && ((*it).src() == src)) 
	    historyBrowser->append(QString("%1%2: %3") .arg((*it).time()) .arg((*it).user()) .arg((*it).message()));
}

