/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Fri Dec 10 18:42:00 EST 2004
    copyright        : (C) 2004-2006 by Sergey Litvinov
    email                : linch@adsl.by
 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  		   *
 *   it under the terms of the GNU General Public License as published by  		   *
 *   the Free Software Foundation; either version 2 of the License, or     		   *
 *  (at your option) any later version.                                   				   *
 ***************************************************************************/

#include <qdir.h>
#include <qtranslator.h>
#include <qhostaddress.h>

#ifdef _WIN32
#include <winsock.h>
#else
#include <sys/socket.h>
#include <netinet/in.h>
#endif

#include "trixapp.h"
#include "global.h"
#include "icons.h"
#include "settings.h"
#include "packet.h"

int main( int argc, char ** argv )
{
    int sd;
    struct sockaddr_in saddr;
    
    srandom(time(NULL));
    
    TrixApp app(argc, argv);    
    
    QString datapath(DATADIR);            
    
    Settings *settings = new Settings();
    settings->load();
	
    if (argc > 1)
    {
	if(QString::compare(argv[1], "-s") == 0)
	{
	    if (argc > 3)
	    {
		QCString files(argv[3]);
		QHostAddress addr = settings->getIP();
		Q_UINT16 port = settings->getPort();
		Packet p;
		int opt = 1;
	    
		if (argc > 3)
		{
		    for(int i = 4; i < argc; i++)
		    {
			files.append(";");
			files.append(argv[i]);
		    }
		}
	    
		p.init();	
		p.add('S');
		p.add(QCString(argv[2]));
		p.add(QCString(files));
	    
		sd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
		setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(int));
	    
		memset(&saddr, 0, sizeof(saddr));
		saddr.sin_family = AF_INET;
		saddr.sin_addr.s_addr = htonl(addr.toIPv4Address());
		saddr.sin_port = htons(port);
		sendto(sd, p.data(), p.size(), 0, (struct sockaddr *)&saddr, sizeof(saddr));
	    
		close(sd);
		delete settings;
		return 0;
	    }
	}
	printf("TriX v%s\n", VERSION);
	printf("Usage:\n");
	printf("%s [-s user=<username> files=<filenames>]\n\n", argv[0]);
	printf("Flags:\n");
	printf("-s\t Send files to user\n");
	    
	delete settings;
	return 0;
    }
	
    Icons::loadIcons();    

    QTranslator translator( 0 );
    translator.load( datapath + "tr/" + settings->getLocal(), "." );
    app.installTranslator( &translator );
    
    QDir d = QDir::home();
    if(!d.cd(".trix")) d.mkdir(".trix");
    
    QMimeSourceFactory::defaultFactory()->addFilePath(QDir::homeDirPath() + "/.trix/avatars/");
    QMimeSourceFactory::defaultFactory()->addFilePath(QDir::homeDirPath() + "/.trix/cache/");
    QMimeSourceFactory::defaultFactory()->addFilePath(datapath + "gfx/");
    QMimeSourceFactory::defaultFactory()->addFilePath(datapath + "avatars/");	
	
    app.setup(settings);
	
    return app.exec();
}
