/****************************************************************************
                          mainform.ui.h  -  description
                             -------------------
    begin                : Mon Dec 20 12:16:00 EST 2004
    copyright            : (C) 2004-2007 by Sergey Litvinov
    email                : linch@adsl.by
 ****************************************************************************/

/****************************************************************************
*   This program is free software; you can redistribute it and/or modify                                                 *
*   it under the terms of the GNU General Public License as published by  		      *
*   the Free Software Foundation; either version 2 of the License, or     		      *
*   (at your option) any later version.                                   				      *
*****************************************************************************/
#include <iostream>

#define ID_CLEAR 50
#define MASS_MSG true

enum {
    DONT_SHOW = 0,
    SHOW_IN_MAIN,
    SHOW_IN_CURRENT
};

enum {
    NORMAL_SHOWN = 0,
    OWN_SHOWN,
    NORMAL_HIDDEN,
    OWN_HIDDEN
};

const QString MainForm::nowplaying()
{
    QString str;
    if (settings->getUseAmaroK())
    {
	if ( !npproc->start() ) 
	    return QString("");

	for (uint i = 0; i < 1000; i++) 
	{
	    if (npproc->canReadLineStdout())
	    {
		str = QString::fromLocal8Bit(npproc->readLineStdout());
		break;
	    }
	    if (npproc->canReadLineStderr())
		return QString("");

	    sleep(1);
	}
    }
    else
    {
	QFile file(settings->getNPFile());
	if (file.open(IO_ReadOnly))
	{
	    str = QString::fromUtf8(file.readAll().data(), file.size());
	    file.close();
	    str.remove('\n');
	    str.remove('\r');
	}
    }
    if (str.length() > 1)
    {
	QUrl::decode(str);
                return tr("Now Playing: ") + str;
    }
    else 
                return QString("");
}

void MainForm::AboutSlot()
{
    AboutForm *af = new AboutForm(this, "About", true);
    af->exec();
}

void MainForm::setup(Settings *s)
{
    updatetimer = new QTimer(this);
    connect(updatetimer, SIGNAL(timeout()), this, SLOT(usersListUpdate()));

    bimgtimer = new QTimer(this);
    connect(bimgtimer, SIGNAL(timeout()), this, SLOT(updateBoard()));
    
    board = NULL;
    xxx = NULL;

    settings = s;
    loadSettings();

    restoreGeometry();
    statusBar()->hide();

    QStringList channels;

    exitreq = false;
    executing = false;

    smlocked = false;
    hideownbmsgs = false;

    autoaway = false;
    awaytimer = 0;
    setMouseTracking ( true );

    status_lut << tr("Active") << tr("DND") << tr("Away") << tr("Offline");
    ractivity << tr("Active") << tr("Offline") << tr("Speaking") << tr("Connected");

    cmdlist.append(ChatCmd("normal", tr("Enter to \"Active\" mode"), CMD_NORMAL, 0));
    cmdlist.append(ChatCmd("away", tr("Enter to \"Away\" mode"), CMD_AWAY,  1));
    cmdlist.append(ChatCmd("offline", tr("Enter to \"Offline\" mode"), CMD_OFFLINE, 1));
    cmdlist.append(ChatCmd("dnd", tr("Enter to \"DND\" mode"), CMD_DND,  1));
    cmdlist.append(ChatCmd("beep", tr("Send signal to user"), CMD_BEEP,  1));
    cmdlist.append(ChatCmd("here", tr("Who is here?"), CMD_HERE,  0));
    cmdlist.append(ChatCmd("info", tr("Get some information about user"), CMD_INFO,  1));
    cmdlist.append(ChatCmd("join", tr("Join to new channel"), CMD_JOIN,  1));
    cmdlist.append(ChatCmd("j", tr("Short version of the previous command"), CMD_JOIN,  1));
    cmdlist.append(ChatCmd("leave", tr("Leave channel"), CMD_LEAVE,  1));
    cmdlist.append(ChatCmd("mesg", tr("Send message to user"), CMD_MESSAGE, 2));
    cmdlist.append(ChatCmd("m", tr("Short version of the previous command"), CMD_MESSAGE, 2));
    cmdlist.append(ChatCmd("mass", tr("Send mass message"), CMD_MASSMESSAGE, 1));
    cmdlist.append(ChatCmd("nick", tr("Change nickname"), CMD_NICK,  1));
    cmdlist.append(ChatCmd("topic", tr("Show/change topic"), CMD_TOPIC,  1));
    cmdlist.append(ChatCmd("add", tr("Add string to topic"), CMD_ADD,  1));
    cmdlist.append(ChatCmd("me", tr("Send message starts with your nickname"), CMD_ME,  1));
    cmdlist.append(ChatCmd("chat", tr("Open private chat"), CMD_CHAT,  1));
    cmdlist.append(ChatCmd("exec", tr("Execute programm and show result (with '-o' option result will send to chat)"), CMD_EXEC, 2));
    cmdlist.append(ChatCmd("quit", tr("Quit trix"), CMD_QUIT,  0));
    cmdlist.append(ChatCmd("help", tr("Show this help"), CMD_HELP,  0));
    cmdlist.append(ChatCmd("?", tr("Short version of the previous command"), CMD_HELP,  0));
    cmdlist.append(ChatCmd("test", tr("Development feature"), CMD_TEST,  1));

    log = new Logs(settings->getHistoryFilePath(), settings->getMaxHistSize(), false);

    if (!Sound::init(settings))
	QMessageBox::warning(this, "TriX", Sound::error(), "OK");
    
    availchannels = new ChannelsListDlg();
    connect(availchannels, SIGNAL(refresh()), this, SLOT(refreshChList()));
    connect(availchannels, SIGNAL(join(const QString &)), this,SLOT(joinChannel(const QString &)));

    if (!settings->getStartMin() && settings->getShowRA()) winstate = WIN_ACTIVE;
    else winstate = WIN_NOTACTIVE;
    
    if (settings->getKeepStatus())
    {
	status = settings->getStatus();
	switch(status)
	{
	case STATUS_DND:
	    autoanswer = settings->getDNDAnswer();
	    break;
	case STATUS_AWAY:
	    autoanswer = settings->getAwayAnswer();
	    break;
	case STATUS_OFFLINE:
	    autoanswer = settings->getOfflineAnswer();
	    break;
	default: break;
	}
    }
    else status = STATUS_NORMAL;
    
    xxx = new XXXProtocol(this, (settings->getEncoding() != "") ?
			  QTextCodec::codecForName(settings->getEncoding()) : QTextCodec::codecForLocale(),
			  settings->getUUID(),
			  settings->getPKey(),
			  settings->getUseUTF());
    
    setupNicksMenu();
    updateNicksMenu();
    
    setupSystray();
    
    setupToolTips();
    
    servicemenupath = QDir::homeDirPath() + "/.kde/share/apps/konqueror/servicemenus/trix.desktop";
    
    requested.append(XXXProtocol::Uuid(settings->getUUID())); //???????????
    
    userslist->setOwnNick(nick);
    
    connect(xxx, SIGNAL(sig_xxx_say(const QString &, const QString &, const QString &)), this, SLOT(on_xxx_say(const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_me(const QString &, const QString &, const QString &)), this, SLOT(on_xxx_me(const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_whohere(const QHostAddress &, const QString &)), this, SLOT(on_xxx_whohere(const QHostAddress &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_imhere(const QHostAddress &, const QString &, const QString &, char, char)), this, SLOT(on_xxx_imhere(const QHostAddress &, const QString &, const QString &, char, char)));
    connect(xxx, SIGNAL(sig_xxx_imhere(const QHostAddress &, const QString &, const QString &, char, char, char, const QString&, const QString &, const QString &)), this, SLOT(on_xxx_imhere(const QHostAddress &, const QString &, const QString &, char, char, char, const QString&, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_join(const QHostAddress &, const QString &, const QString &, char, char)), this, SLOT(on_xxx_join(const QHostAddress &, const QString &, const QString &, char, char)));
    connect(xxx, SIGNAL(sig_xxx_join(const QHostAddress &, const QString &, const QString &, char, char, const QString &, char, const QString &, const QString &)), this, SLOT(on_xxx_join(const QHostAddress &, const QString &, const QString &, char, char, const QString &, char, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_leave(const QString &, const QString &, char)), this, SLOT(on_xxx_leave(const QString &, const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_private_join(const QString &, const QString &, char)), this, SLOT(on_xxx_private_join(const QString &, const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_private_leave(const QString &, const QString &, char)), this, SLOT(on_xxx_private_leave(const QString &, const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_private_say(const QString &, const QString &, const QString &)), this, SLOT(on_xxx_private_say(const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_private_me(const QString &, const QString &, const QString &)), this, SLOT(on_xxx_private_me(const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_message(const QHostAddress &, const QCString &, const QString &, const QString &, const QString &)), this, SLOT(on_xxx_message(const QHostAddress &, const QCString &, const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_massmessage(const QHostAddress &, const QCString &, const QString &, const QString &, const QString &)), this, SLOT(on_xxx_massmessage(const QHostAddress &, const QCString &, const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_messageack(char, const QString &, const QString &, char, const QString &)), this, SLOT(on_xxx_messageack(char, const QString &, const QString &, char, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_messageack(const QCString &, char, const QString &, const QString &, char, const QString &)), this, SLOT(on_xxx_messageack(const QCString &, char, const QString &, const QString &, char, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_herereq(const QHostAddress &, const QString &, const QString &)), this, SLOT(on_xxx_herereq(const QHostAddress &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_hereack(const QString &, const QString &, const QString &, char )), this, SLOT(on_xxx_hereack(const QString &, const QString &, const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_inforeq(const QHostAddress &, const QString &, const QString &)), this, SLOT(on_xxx_inforeq(const QHostAddress &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_inforeqack(const QHostAddress &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, bool, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &)), this, SLOT(on_xxx_inforeqack(const QHostAddress &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, bool, const QString &,  const QString &, const QString &, const QString &, const QString &, const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_changeactive(const QString &, char)), this, SLOT(on_xxx_changeactive(const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_reqchannels(const QHostAddress&, const QString &)), this, SLOT(on_xxx_reqchannels(const QHostAddress&, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_reqchannelsack(const QHostAddress&, const QString &, const QString&)), this, SLOT(on_xxx_reqchannelsack(const QHostAddress&, const QString &, const QString&)));
    connect(xxx, SIGNAL(sig_xxx_nickchange(const QString &, const QString &, char)), this, SLOT(on_xxx_nickchange(const QString &, const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_avatarchange(const QString &, const QString &, char)), this, SLOT(on_xxx_avatarchange(const QString &, const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_changestatus(const QString &, const QString &, char, char, const QString &)), this, SLOT(on_xxx_changestatus(const QString &, const QString &, char, char, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_currenttopic(const QHostAddress &, const QString &, const QString &, const QString &)), this, SLOT(on_xxx_currenttopic(const QHostAddress &, const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_newtopic(const QHostAddress &, const QString &, const QString &)), this, SLOT(on_xxx_newtopic(const QHostAddress &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_ping(const QHostAddress &, const QString &, const QString &, const QString &, const QCString &)), this, SLOT(on_xxx_ping(const QHostAddress &, const QString &, const QString &, const QString &, const QCString &)));
    connect(xxx, SIGNAL(sig_xxx_pong(const QHostAddress &, const QString &, const QString &, const QString &, const QCString &)), this, SLOT(on_xxx_pong(const QHostAddress &, const QString &, const QString &, const QString &, const QCString &)));
    connect(xxx, SIGNAL(sig_xxx_floodnotify(const QString &, const QString &, const QString &)), this, SLOT(on_xxx_floodnotify(const QString &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_beep(const QHostAddress &, const QString &, const QString &)), this, SLOT(on_xxx_beep(const QHostAddress &, const QString &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_beepack(const QString &, const QString &, char)), this, SLOT(on_xxx_beepack(const QString &, const QString &, char)));
    connect(xxx, SIGNAL(sig_xxx_avatarrequest(const QHostAddress &, const QString &)), this, SLOT(on_xxx_avatarrequest(const QHostAddress &, const QString &)));
    connect(xxx, SIGNAL(sig_xxx_whohereneeded(const QHostAddress &)), this, SLOT(on_xxx_whohereneeded(const QHostAddress &)));
    connect(xxx, SIGNAL(sig_xxx_boardmsgupd(const QHostAddress &, const QString &, Q_UINT32)), this, SLOT(boardMsgUpdated(const QHostAddress &, const QString &, Q_UINT32)));
    connect(xxx, SIGNAL(sig_xxx_boardmsgdel(const QHostAddress &, const QString &, Q_UINT32)), this, SLOT(boardMsgDeleted(const QHostAddress &, const QString &, Q_UINT32)));
    connect(xxx, SIGNAL(sig_activityDetected(const QHostAddress &)), this, SLOT(resetActivityTimer(const QHostAddress &)));
    connect(xxx, SIGNAL(sig_xxx_sendtoprocess(const QHostAddress &, const QString &, const QString &)), this, SLOT(on_xxx_sendtoprocess(const QHostAddress &, const QString &, const QString &)));
    
    connect(xxx, SIGNAL(sig_tcpFileRecvFinished(const QString&, const QString&)), this, SLOT(tcpFileRecvFinished(const QString&, const QString&)));
    connect(xxx, SIGNAL(sig_httpFileRecvFinished(const QString&, const QString&)), this, SLOT(httpFileRecvFinished(const QString&, const QString&)));
    connect(xxx, SIGNAL(sig_tcpAvatarRecvFinished(const QString&, const QString&)), this, SLOT(tcpAvatarRecvFinished(const QString&, const QString&)));
    connect(xxx, SIGNAL(sig_tcpFileSendFinished(const QString&, const QString&)), this, SLOT(tcpFileSendFinished(const QString&, const QString&)));
    connect(xxx, SIGNAL(sig_tcpMessageACKReceived(const QString&, const QString&, char)), this, SLOT(tcpMessageACKReceived(const QString&, const QString&, char)));
    connect(xxx, SIGNAL(sig_tcpPvtMsgReceived(const QString&, const QString&)), this, SLOT(tcpPvtMsgReceived(const QString&, const QString&)));
    connect(xxx, SIGNAL(sig_tcpRecvChatMessage(const QHostAddress&, const QString&)), this, SLOT(tcpRecvChatMessage(const QHostAddress&, const QString&)));
    connect(xxx, SIGNAL(sig_tcpChatSessionEstablished(const QString&)), this, SLOT(tcpChatSessionEstablished(const QString&)));
    connect(xxx, SIGNAL(sig_tcpCloseChatSession(const QString&)), this, SLOT(tcpCloseChatSession(const QString&)));
    connect(xxx, SIGNAL(sig_tcpRecvTypingMode(const QString&, uint)), this, SLOT(tcpRecvTypingMode(const QString&, uint)));
    connect(xxx, SIGNAL(sig_addToBoard(BoardMsg *)), this, SLOT(addToBoard(BoardMsg *)));

    //
    // Initializing DCOP request to amaroK player
    //
    npproc = new QProcess(this);
    npproc->addArgument("dcop");
    npproc->addArgument("amarok");
    npproc->addArgument("player");
    npproc->addArgument("nowPlaying");

    //
    // Reference to the process which will be calling with "/exec"command
    //
    proc = new QProcess(this);
    connect(proc, SIGNAL(readyReadStdout()), this, SLOT(procDataReady()));    

    //
    // Initialize timer for users activity monitoring
    //
    acttimer = new QTimer(this);
    connect(acttimer, SIGNAL(timeout()), this, SLOT(actTimerEvent()));
    acttimer->start(ACT_TIMER_INTERVAL);
    
    smtimer = new QTimer(this);
    connect(smtimer, SIGNAL(timeout()), this, SLOT(updateServiceMenu()));
    
    btimer = new QTimer(this);
    connect(btimer, SIGNAL(timeout()), this, SLOT(updateBoard()));
    
    reconfigureNetwork();
    //
    // Show greeting message
    //
    greeting();
    //
    // Open favourite channels on start
    //
    if (settings->getStartChannels())
    {
	channels = QStringList::split(",", settings->getChannelsList());
	for (uint i = 0; i < channels.count(); i++)
	    joinChannel(channels[i]);
    }
    chattab->setCurrentPage(settings->isBoardEnabled() ? 1 : 0);
    
    chatline->setFocus();
}

void MainForm::greeting()
{
    chattab->write(MAIN_CHANNEL, tr("// Welcome to TriX v") + QString(VERSION) + ", " + nick + "!", QColor(0x00, 0xff, 0x80));
    chattab->write(MAIN_CHANNEL, tr("// released under terms of GNU General Public License (GPL)"), QColor(0x00, 0xff, 0x80));
    chattab->write(MAIN_CHANNEL, tr("// send comments to <a href=\"#Elinch@adsl.by\">linch@adsl.by</a>"), QColor(0x00, 0xff, 0x80));
    chattab->write(MAIN_CHANNEL, "<hr>", QColor(0x00, 0xff, 0x80));

    connect(chattab, SIGNAL(sig_linkClicked(const QString &, const QString &)), this, SLOT(on_linkClicked(const QString &, const QString &)));
    connect(chattab, SIGNAL(sig_topic_clear(const QString &)), this, SLOT(on_clearTopic(const QString &)));
    connect(chattab, SIGNAL(keypressed(const QString &)), this, SLOT(on_chattab_keypressed(const QString &)));
    connect(chattab, SIGNAL(updateCaption()), this, SLOT(updateCaptionSlot()));
    connect(chattab, SIGNAL(saveChatLineText(const QString &)), this, SLOT(saveChatLine(const QString &)));
    connect(chattab, SIGNAL(loadChatLineText(const QString &)), this, SLOT(loadChatLine(const QString &)));
    connect(chattab, SIGNAL(boardActivated()), this, SLOT(boardActivated()));
    connect(chattab, SIGNAL(boardDisactivated()), this, SLOT(boardDisactivated()));
}

void MainForm::actTimerEvent()
{
    if (!settings->getLegacyStyle()) userslist->incActivityTimer();	//by 5 seconds for each user, except local user
    incAwayTimer(); //by 5 seconds for local user
}

void MainForm::incAwayTimer()
{
    awaytimer += 5;
    if ((awaytimer == settings->getAwayLimit()) && (status == STATUS_NORMAL))
    {
	autoanswer = tr("User automatically changes mode to \"Away\"");
	xxx->xxx_changestatus(nick, status = STATUS_AWAY, gender, autoanswer);
	trayicon = Icons::icon_user_away;
	tray->setIcon(trayicon.pixmap());
	autoaway = true;
    }
}

void MainForm::setupNicksMenu()
{
    nicksmenu = new QPopupMenu(this);
    connect(nicksmenu, SIGNAL(activated(int)), this, SLOT(nicksMenuAction(int)));
}

void MainForm::nicksMenuAction(int opt)
{
	if (opt == ID_CLEAR)
	{
		nicksmap.clear();
		updateNicksMenu();
		return;
	}
	if ((opt >= 0) && (opt < ID_CLEAR)) changeNick(nickslist[opt]);
}

void MainForm::updateNicksMenu()
{
    nicksmenu->clear();

    nickslist.clear();
    
    if(nicksmap.isEmpty()) nicksmap.insert(nick, QString(QChar(gender) + userpixmap));

    uint i = 0;
    for (NicksMap::iterator it = nicksmap.begin(); it != nicksmap.end(); it++)
    {
	QString nickname = it.key();
	nickname.replace("&", "&&");
	nicksmenu->insertItem(nickname, i);
	nicksmenu->setAccel(QString("Alt+" + QString::number(i + 1)), i);
	nickslist.append(it.key());
	i++;
    }
    nicksmenu->insertSeparator();
    nicksmenu->insertItem(tr("Clear"), ID_CLEAR);	
}

void MainForm::setupSystray()
{
    systraymenu = new QPopupMenu(this);
    systraymodemenu = new QPopupMenu(this);
    
    systraymodemenu->insertItem(status_lut[0], this, SLOT(goNORMAL()));
    systraymodemenu->insertItem(Icons::icon_user_dnd, status_lut[1], this, SLOT(goDND()));
    systraymodemenu->insertItem(Icons::icon_user_away, status_lut[2], this, SLOT(goAWAY()));
    systraymodemenu->insertItem(Icons::icon_user_offline, status_lut[3], this, SLOT(goOFFLINE()));
    systraymenu->insertItem(tr("Chat mode"), systraymodemenu);
    systraymenu->insertSeparator();
    systraymenu->insertItem(Icons::icon_settings, tr("Preferences"), this, SLOT(PreferencesSlot()));
    systraymenu->insertSeparator();
    systraymenu->insertItem(Icons::icon_exit, tr("Exit"), this, SLOT(closeSlot()));

    tray = new TrayIcon(this);

    switch (status)
    {
	case STATUS_NORMAL: 
	    trayicon = Icons::icon_common;
	    break;
	case STATUS_DND:
	    trayicon = Icons::icon_user_dnd;
	    break;
	case STATUS_AWAY:
	    trayicon = Icons::icon_user_away;
	    break;
	case STATUS_OFFLINE:
	    trayicon = Icons::icon_user_offline;
	    break;
	default: 
	    break;
    }
    tray->setIcon(trayicon.pixmap());
    tray->setPopup(systraymenu);
    connect(tray, SIGNAL(clicked(const QPoint&,int)), this, SLOT(systrayClicked()));
    tray->show();
    tray->setWMDock(true);
}

void MainForm::systrayClicked()
{
    tray->setIcon(trayicon.pixmap());
    if (isMinimized())
	showNormal(); 
    else
	setShown(!isShown());
}

void MainForm::setupToolTips()
{
    QToolTip::remove(userslist);
    new ListViewToolTip(userslist);
}

void MainForm::hideEvent(QHideEvent *event)
{
    if (settings->getShowRA()) xxx->xxx_changeactive(nick, winstate = WIN_NOTACTIVE);
}

void MainForm::showEvent(QShowEvent *event)
{
    if (settings->getShowRA()) xxx->xxx_changeactive(nick, winstate = WIN_ACTIVE);
}

void MainForm::windowActivationChange ( bool oldActive )
{
    if (!oldActive) chatline->setFocus();
}

void MainForm::finishSession()
{
    //
    // Leaving all existing channels
    //
    QStringList chlist = QStringList::split("#", chattab->channelsList());
    QStringList pvtlist = chattab->privatesList();
    for (uint i  = 0; i < chlist.count(); i++) 
	leaveChannel(QString("#" + chlist[i]));
    for (uint i  = 0; i < pvtlist.count(); i++) 
	leaveChannel(pvtlist[i]);
    
    xxx->xxx_leave(nick, MAIN_CHANNEL, gender);
    
    Sound::close();

    saveSettings();
    
    QFile::remove (servicemenupath);
    
    exitreq = true;
    
    delete settings;
    delete availchannels;
    delete log;
    delete board;
}

void MainForm::closeSlot()
{
    finishSession();
           
    close();
}

void MainForm::saveSettings()
{
    QStringList lastnicks;
    settings->setGeometry(x(), y(), width(), height(), rightframe->width());
    settings->setMultiline(chatline->isMultiline());
    settings->setStatus(autoaway ? STATUS_NORMAL : status);
    settings->setNick(nick);
    settings->setGender(gender);
    settings->setUserPixmap(userpixmap);
    for (NicksMap::iterator it = nicksmap.begin(); it != nicksmap.end(); it++)
    {
	lastnicks.append(it.key());
	lastnicks.append(it.data());
    }
    settings->setLastNicks(lastnicks.join(":"));
}

void MainForm::closeEvent(QCloseEvent *event)
{
    if (exitreq) 
	event->accept();
    else
	setShown(false);
}

void MainForm::on_chattab_keypressed(const QString & s)
{
    chatline->setFocus();
    chatline->insert(s);
}

const QString MainForm::timestring()
{
    QTime current = QTime::currentTime();
    if (settings->getTimeStampWithSeconds()) 
	return QString("[") + current.toString("hh:mm:ss") + QString("] ");
    else
	return QString("[") + current.toString("hh:mm") + QString("] ");
}

const QString MainForm::datestring()
{
    QDate current = QDate::currentDate();
    return QString("[") + current.toString("dd.MM.yyyy") + QString("] ");
}

void MainForm::on_linkClicked(const QString &str, const QString &chn)
{
    bool ok;
    QString link;
	
    QProcess *process;
	
    if (str[0] == '#') 
    {
	link = str.right(str.length() - 2);
		
	int cmd = str[1];
	if (chattab->isBoard(chn))
	{
	    BItemMap::iterator it;
	    switch (cmd)
	    {
	    case 'R':
		if (board) 
		{
		    board->delBoardMsg(link.toUInt(&ok, 36));
		    updateBoard();
		}
		break;
	    case 'O':
		PreferencesSlot();
		break;
	    case 'A':
		onUsersListDoubleClicked(getNick(xxx->getIP(link)));
		break;
	    case 'h':
		if (bimap[link] == NORMAL_SHOWN)
		    bimap[link] = NORMAL_HIDDEN;
		else if (bimap[link] == NORMAL_HIDDEN)
		    bimap[link] = NORMAL_SHOWN;
		if (bimap[link] == OWN_SHOWN)
		    bimap[link] = OWN_HIDDEN;
		else if (bimap[link] == OWN_HIDDEN)
		{
		    bimap[link] = OWN_SHOWN;
		    hideownbmsgs = false;
		}
		updateBoard();
		break;
	    case 'S':
		for (it = bimap.begin(); it != bimap.end(); it++)
		{
		    if (it.data() == NORMAL_HIDDEN)
			it.data() = NORMAL_SHOWN;
		    if (it.data() == OWN_HIDDEN)
			it.data() = OWN_SHOWN;
		}
		updateBoard();
		break;
	    case 'H':
		for (it = bimap.begin(); it != bimap.end(); it++)
		{
		    if (it.data() == NORMAL_SHOWN)
			it.data() = NORMAL_HIDDEN;
		    if (it.data() == OWN_SHOWN)
			it.data() = OWN_HIDDEN;
		}
		updateBoard();
		break;
	    case 'M':
		hideownbmsgs = !hideownbmsgs;
		if (hideownbmsgs)
		{
		    for (it = bimap.begin(); it != bimap.end(); it++)
		    {
			if (it.data() == OWN_SHOWN)
			    it.data() = OWN_HIDDEN;
		    }
		} else 
		{
		    for (it = bimap.begin(); it != bimap.end(); it++)
		    {
			if (it.data() == OWN_HIDDEN)
			    it.data() = OWN_SHOWN;
		    }
		}
		updateBoard();
		break;
	    case 'L':
		process = new QProcess(settings->getWebBrowser(), NULL);
		QUrl::decode(link);
		link.replace("%3A", ":");
		process->addArgument(link);
		process->launch("");
		delete process;
		break;
	    default: break;
	    }
	    return;
	}
	switch (cmd) 
	{
	case 'N':
	    chatline->sayNick(link);
	    if (link.left(3) == "/m ") link = link.right(link.length() - 3);
	    userslist->scrollToUser(link);
	    break;
				
	case 'L':
	    process = new QProcess(settings->getWebBrowser(), NULL);
	    QUrl::decode(link);
	    if (link.startsWith("http://")) link = link.right(link.length() - 7);
	    if (settings->getWebBrowser().contains("konqueror")) link = QString::fromUtf8(QTextCodec::codecForLocale()->fromUnicode(link));
	    QUrl::encode(link);
	    link.prepend("http://");
	    std::cout << link << std::endl;
	    process->addArgument(link);
	    process->launch("");
	    delete process;
	    break;
		    
	case 'T':
	    process = new QProcess(settings->getFTPClient(), NULL);
	    QUrl::decode(link);
	    if (link.startsWith("ftp://")) link = link.right(link.length() - 6);
	    if (settings->getFTPClient().contains("konqueror")) link = QString::fromUtf8(QTextCodec::codecForLocale()->fromUnicode(link));
	    QUrl::encode(link);		   
	    link.prepend("ftp://");
	    process->addArgument(link);
	    process->launch("");
	    delete process;
	    break;

	case 'S':
	    if (link.startsWith("\\\\"))
	    {
		std::cout << link << std::endl;
		link = link.right(link.length() - 2);
		std::cout << link << std::endl;
		QUrl::decode(link);
		process = new QProcess(settings->getSMBBrowser(), NULL);
		if (settings->getSMBBrowser().contains("konqueror")) link = QString::fromUtf8(QTextCodec::codecForLocale()->fromUnicode( link ));
		QUrl::encode(link);
		link.prepend("smb://");
		std::cout << link << std::endl;
		process->addArgument(link);
		process->launch("");
		delete process;
	    }
	    break;
	
	case 'C':
	    joinChannel(link);
	    break;
	
	case 'E':
	    process = new QProcess(settings->getEmailClient(), NULL);
	    link = QString::fromUtf8(QTextCodec::codecForLocale()->fromUnicode( link ));
	    QUrl::encode(link);
	    link.prepend("mailto:");
	    process->addArgument(link);
	    process->launch("");
	    delete process;
	    break;
	default: break;
	}
    }
}

void MainForm::beep(const QString &to)
{
    xxx->xxx_beep(userslist->getIP(to), to, nick);
}

void MainForm::sendLine()
{
    QString str = chatline->text();
    if (!str.isEmpty())
    {
	if (str[0] == '/')
	{
	    if (!cmdparse(str))
		chattab->write(MAIN_CHANNEL, timestring() + tr("Unknown command"), settings->getSystemColor());
	}
	else
	{
	    if (chattab->isPrivate())
	    {
		const QString to = chattab->current();
		if (userslist->exists(to))
		{
		    xxx->xxx_private_say(userslist->getIP(to), nick, to, str);
		    if (to != nick)
		    {
			msgparse(str);

			tmp = QString("%1&lt;<a href=\"#N%2\">%3</a>&gt;: %4")
			      .arg(timestring(), urlenc(nick), qtparse(nick), str);
			chattab->write(to, tmp, settings->getOwnColor());
			log->write(datestring(), timestring(), to, PRV_TYPE, nick, str);
			if ((status == STATUS_NORMAL)  && settings->isSoundEnabled(Sound::Chatline)) Sound::play(Sound::Chatline);
		    }
		}
		else
		{
		    tmp = QString(tr("%1Remote side disconnected")).arg(timestring());
		    chattab->write(chattab->current(), tmp, settings->getSystemColor());
		}
	    }
	    else
		xxx->xxx_say(chattab->current(), nick, str);
	}
    }
    chatline->clear();
}

void MainForm::addToTrayToolTip(const QString& tt)
{
    traytooltip.append("<p>" + tt + "</p>");
    if (traytooltip.count() > (uint)settings->getMaxLinesCount())
	traytooltip.erase(traytooltip.begin());
    tray->setToolTip(traytooltip.join(""));
}

void MainForm::traynotify()
{
    if (settings->isSystrayNotifyEnabled())
    {
	if (!isShown())
	    tray->setIcon(Icons::icon_traynotify.pixmap());
    }
}

void MainForm::openAvatarsPage()
{
    openPreferences(1);  // 1 - first page
    UserInfoDlg *u = (UserInfoDlg*)sender();
    if (u)
    {
	u->setPhoto(getLocalAvatar());
	u->raise();
    }
}

void MainForm::PreferencesSlot()
{
    openPreferences(4); // 4 - fourth page
}

void MainForm::openPreferences(int page)
{
    saveSettings();
    
    SettingsDlg *sd = new SettingsDlg(this, "Preferences", true);
    sd->setup(settings, page);
    connect(sd, SIGNAL(applySettings(bool)), this, SLOT(applySettingsSlot(bool)));
    sd->exec();
}

void MainForm::applySettingsSlot(bool c)
{
    QString oldnick = nick, newnick;
    QString oldpixmap = userpixmap;
  
    updatetimer->stop();
    
    loadSettings();
    
    log->setPath(settings->getHistoryFilePath());
    log->setMaxSize(settings->getMaxHistSize());

    updateNicksMenu();

    if (oldnick != nick) 
    {
	newnick = nick;
	nick = oldnick;
	changeNick(newnick);
    }

    if (oldpixmap != userpixmap) changeAvatar(userpixmap);
    
    if (c) 
    {
	reconfigureNetwork();
	forceRefresh();
    }
    else
	imhereToAll();
}

void MainForm::restoreGeometry()
{
    resize(settings->getWidth(), settings->getHeight());

    if ((settings->getX() >= 0) && (settings->getY() >= 0))
	move(settings->getX(), settings->getY());

    rightframe->resize(settings->getFrameWidth(), rightframe->height());
    mainsplitter->setResizeMode(rightframe, QSplitter::KeepSize);
    
    chatline->setMultiline(settings->getMultiline());
    if (settings->getMultiline()) editMultilineAction->toggle();
}

void MainForm::loadSettings()
{
    QStringList lastnicks;
    
    chatline->setFont(settings->getEditLineFont());
    userslist->setFont(settings->getUsersListFont());
   
    nickedit->setPaletteBackgroundColor(settings->getNickEditBackColor());
    nickedit->setPaletteForegroundColor(settings->getNickEditTextColor());
    userslist->setPaletteBackgroundColor(settings->getUsersListBackColor());
    userslist->setPaletteForegroundColor(settings->getUsersListTextColor());
    chatline->setPaletteBackgroundColor(settings->getChatLineEditBackColor());
    chatline->setPaletteForegroundColor(settings->getChatLineEditTextColor());
   
    gender = settings->getGender();
    
    userpixmap = settings->getUserPixmap();
   
    nick = settings->getNick();
    
    lastnicks = QStringList::split(":", settings->getLastNicks());
    if (lastnicks.count() & 0x01) lastnicks.append("0male.png");
    nicksmap.clear();
    for(uint i = 0; i < lastnicks.count(); i += 2)
	nicksmap.insert(lastnicks[i], lastnicks[i + 1]);

    nickedit->setText(nick);
   
    userslist->updatePixmaps();
    
    if (settings->getAutoUpdate())
	if (settings->getAutoUpdateInterval() > 0) updatetimer->start(settings->getAutoUpdateInterval() * 1000, false);
    
    if (board)
    {
	if (!settings->isBoardEnabled()) closeBoard();
    }
    else
	if (settings->isBoardEnabled()) openBoard();
}

void MainForm::openBoard()
{
    brlock = true;
    bimglock = false;
    chattab->addBoard(settings->getChatFont());
    board = new Logs(QDir::homeDirPath() + "/.trix/board/board", 0, true);
    
    QTimer::singleShot(1000, this, SLOT(updateBoard()));
    QTimer::singleShot(30000, this, SLOT(releaseBoardLock()));
    //TODO make and connect signals and slots
}

void MainForm::closeBoard()
{
    chattab->delBoard();
    delete board;
    board = NULL;
}

QString MainForm::qtparse(const QString &n)
{
    QString p = n;
    
    p.replace('<', "&lt;");
    p.replace('>', "&gt;");
    
    return p;
}

QString MainForm::urlenc(const QString &n)
{
    QString e = n;
    QUrl::encode(e);
    return e;
}

void MainForm::on_xxx_say(const QString &channel, const QString &from, const QString &text)
{
    QString msg = text;
    QColor clr;
    
    if (chattab->exists(channel))
    {
	msgparse(msg);
	tmp = QString("%1&lt;<a href=\"#N%2\">%3</a>&gt;: %4")
	      .arg(timestring(), urlenc(from),qtparse(from), msg);
	
	if (from == nick)
	    clr = settings->getOwnColor();
	else
	{
	    if (settings->getUseUsersColors())
		clr = userslist->getUserColor(from);
	    else
		clr = settings->getUsersColor();
	}
	
	chattab->write(channel, tmp, clr);
	log->write(datestring(), timestring(), channel, CHN_TYPE, from, msg);
	traynotify();
	addToTrayToolTip(QString("<nobr><b>%1</b>:%2&lt;%3&gt;:</nobr> %4").arg(channel).arg(timestring()).arg(from).arg(qtparse(text)));
	if ((status == STATUS_NORMAL)  && settings->isSoundEnabled(Sound::Chatline) && (QString::compare(chattab->current(), channel)==0))
	Sound::play(Sound::Chatline);
    }
}

void MainForm::on_xxx_me(const QString &channel, const QString &from, const QString &text)
{
    tmp = QString("%1*%2 %3").arg(timestring(), from, text);
    
    chattab->write(channel, tmp, settings->getSystemColor());
  
    log->write(datestring(), timestring(), channel, chattab->isPrivate(channel) ? PRV_TYPE : CHN_TYPE, from, QString("*%1 %2") .arg(from) .arg(text));
 
    traynotify();
    addToTrayToolTip(QString("<nobr><b>%1:</b> %2*%3</nobr> %4").arg(channel).arg(timestring()).arg(from).arg(qtparse(text)));
}

void MainForm::on_xxx_whohere(const QHostAddress &source, const QString &updater)
{
    xxx->xxx_imhere(source, updater, nick, status, gender, winstate, userpixmap, settings->getOwnColor());
}

void MainForm::on_xxx_imhere(const QHostAddress &source, const QString &updater, const QString &responder, char stat, char ractive)
{
    if (!userslist->exists(responder))
    {
	userslist->addUser(responder, '0', stat, source, QString(""), settings->getUsersColor().name());
	updateServiceMenu();
	    
    } 
    else
	userslist->setUserColor(responder, settings->getUsersColor().name());
    
    userslist->setRactive(responder, ractive);
}

void MainForm::on_xxx_imhere(const QHostAddress &source, const QString &updater, const QString &responder, char stat, char ractive, char gend, const QString &cname, const QString &fname, const QString &bids)
{
    if (!userslist->exists(responder))
    {
	userslist->addUser(responder, gend, stat, source, fname, cname);
	checkAvatar(source, fname);
	updateServiceMenu();
	addBoardMsg(source, bids);
//	requestMissed(source);
    }
    else
	userslist->setUserColor(responder, cname);
    userslist->setRactive(responder, ractive);
}

void MainForm::checkAvatar(const QHostAddress &source, const QString &fname)
{
    if ((!fname.isEmpty()) && (!QFile::exists(QString(QDir::homeDirPath() + "/.trix/avatars/" + fname))) && (!QFile::exists(QString(QString(DATADIR) + "avatars/" + fname)))  && (!QFile::exists(QString(QString(DATADIR) + "gfx/" + fname))))
    {
	std::cout << "Request for " << fname << " from " << source.toString() << std::endl;
	xxx->xxx_avatarrequest(source, fname);
    }
}

void MainForm::on_xxx_join(const QHostAddress &source, const QString &who, const QString &channel, char stat, char gend, const QString& cname, char icon, const QString &fname, const QString &bids)
{
    if (userslist->getUser(who) == NULL) 
    {
	userslist->addUser(who, gend, stat, source, fname, cname);
	checkAvatar(source, fname);
	updateServiceMenu();
	addBoardMsg(source, bids);
//	requestMissed(source);
    }
    if (chattab->exists(channel))
    {
	if (QString::compare(who, nick) != 0)
	{
	    xxx->xxx_imhere(source, who, nick, status, gender, winstate, userpixmap, settings->getOwnColor());
	    xxx->xxx_currenttopic(source, who, channel, chattab->getTopic(channel));
	}
	
	if (settings->getSystemMsgShow() & JOIN_LEAVE_SHOW)
	{
	    // Space at the end needs to translate verbs in non English languages
	    QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("joins")) : QString(tr("joins ")); 
	    tmp = QString("%1<a href=\"#N%2\">%3</a> %4")
		  .arg(timestring(), urlenc(who), qtparse(who), gender_depend);
	    tmp.append(tr(" channel %1 (%2 mode)").arg(qtparse(channel), status_lut[stat - STATUS_NORMAL]));
	      
	    chattab->write(channel, tmp, settings->getJoinLeaveColor());
	    traynotify();
	}
	
	if ((status == STATUS_NORMAL) && (channel == chattab->current()) && settings->isSoundEnabled(Sound::JoinChannel)) 
	    Sound::play(Sound::JoinChannel);
    }
    availchannels->add(channel);    
}

void MainForm::on_xxx_join(const QHostAddress &source, const QString &who, const QString &channel, char stat, char gend)
{
    on_xxx_join(source, who, channel, stat, gend, settings->getUsersColor().name(), 0, "", "");
}

void MainForm::on_xxx_leave(const QString &who, const QString &channel, char gend)
{
    if(settings->getSystemMsgShow() & JOIN_LEAVE_SHOW)
    {
	QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("leaves")) : QString(tr("leaves "));
	tmp  = tr("%1%2 %3 channel %4")
	       .arg(timestring(), qtparse(who), gender_depend, qtparse(channel));

	chattab->write(channel, tmp, settings->getJoinLeaveColor());
    }
	if (channel.lower() == QString(MAIN_CHANNEL).lower()) 
	{
	    xxx->delUser(userslist->getIP(who));
	    userslist->delUser(who);
	    updateServiceMenu();
	}
    
    if ((status == STATUS_NORMAL) && (channel == chattab->current()) && settings->isSoundEnabled(Sound::LeaveChannel))
	Sound::play(Sound::LeaveChannel);
}

void MainForm::on_xxx_private_join(const QString &from, const QString &to, char gend)
{
    const QString prvname = (from == nick) ? to : from;

    if (!chattab->exists(prvname))
	chattab->addPrivate(prvname, settings->getBackgroundColor(), settings->getChatFont());

    QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("joins")) : QString(tr("joins "));
    tmp  = tr("%1<a href=\"#N%2\">%3</a> %4 private")
	   .arg(timestring(), urlenc(from), qtparse(from), gender_depend);
    chattab->write(prvname, tmp, settings->getJoinLeaveColor());
    traynotify();
}

void MainForm::on_xxx_private_leave(const QString &from, const QString &to, char gend)
{
    QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("leaves")) : QString(tr("leaves "));
    const QString prvname = (from == nick) ? to : from;
    tmp = tr("%1%2 %3 private chat")
	  .arg(timestring(), qtparse(from), gender_depend);
    chattab->write(prvname, tmp, settings->getJoinLeaveColor());
}

void MainForm::on_xxx_private_say(const QString &from, const QString &to, const QString &text)
{
    QString msg = text;
    QColor clr;
    const QString prvname = (from == nick) ? to : from;
    
    if (!chattab->exists(prvname)) openPrivate(from);
    
    msgparse(msg);
    tmp = QString("%1&lt;<a href=\"#N%2\">%3</a>&gt;: %4")
	  .arg(timestring(), urlenc(from), qtparse(from), msg);
    
    if (from == nick)
	clr = settings->getOwnColor();
    else
    {
	if (settings->getUseUsersColors())
	    clr = userslist->getUserColor(from);
	else
	    clr = settings->getUsersColor();
    }

    chattab->write(prvname, tmp, clr);
    
    log->write(datestring(), timestring(), prvname, PRV_TYPE, from, msg);
    
    if ((status == STATUS_NORMAL)  && settings->isSoundEnabled(Sound::Chatline) && (chattab->current() == prvname)) Sound::play(Sound::Chatline);
    
    traynotify();
    
    addToTrayToolTip(QString("<nobr><b>%1:</b> %2&lt;%3&gt;:</nobr> %4").arg(prvname).arg(timestring()).arg(from).arg(qtparse(text)));
}

void MainForm::on_xxx_private_me(const QString &from, const QString &to, const QString &text)
{
    if (from != to)
    {
	const QString prvname = (from == nick) ? to : from;
	on_xxx_me(prvname, from, text);
    }
}

void MainForm::findlinks(QString &msg, const QString &substr, const QString &rgs, const QString &type)
{
    QString text, link, str;
    QStringList list, strlist;
    QRegExp rx(substr);
    QRegExp r;
    int pos = 0;
	
    while (pos >= 0)
    {
	pos = rx.search(msg, pos);
	if (pos > -1) 
	{
	    if (!list.contains(rx.cap(1))) 
		list += rx.cap(1);
	    pos += rx.matchedLength();
	}
    }
	
    for (uint i = 0; i < list.count(); i++)
    {
	link = list[i];
	text = list[i];
	r.setPattern(QString(rgs).arg(QRegExp::escape(link)));
	QUrl::encode(link);
	text = qtparse(text);
	QUrl::decode(text);	

	str = QString("<DNP><font color=\"%1\"><a href=\"%2%3\">%4</a></font><DNP>").arg(settings->getPrivateColor().name(), type, link, text);
	
	strlist = QStringList::split("<DNP>", msg, true);
	for (uint t = 0; t < strlist.count(); t += 2)
	    strlist[t].replace(r, str);
	msg = strlist.join("<DNP>");
    }
}

void MainForm::msgparse(QString &msg)
{
    msg.replace("<DNP>", "&lt;DNP&gt;");
    
    for (uint i = 0; smilespack[i].smile; i++) 
    {
	tmp.sprintf("<DNP><img src=\"%s%s%s\"><DNP>", DATADIR, "gfx/", smilespack[i].file);
	msg.replace(smilespack[i].smile, tmp);
    }
  
    msg.append(" ");
    msg.prepend(" ");
    
    findlinks(msg, "\\b(\\S+@\\S+\\.\\S+)\\b", "\\b(%1)\\b", "#E");
    findlinks(msg, "\\b(http\\://\\S+)\\b", "\\b(%1)\\b", "#L");
    findlinks(msg, "\\b(ftp\\://\\S+)\\b", "\\b(%1)\\b", "#T");
    findlinks(msg, "\\b(www\\.\\S+)\\b", "\\b(%1)\\b", "#L");
    findlinks(msg, "(#\\b\\S+)\\b", "(%1)\\b", "#C");
    findlinks(msg, "(\\\\{2}\\S+)\\b", "(?:%1)\\b", "#S");
    
    QStringList strlist = QStringList::split("<DNP>", msg, true);
    for (uint t = 0; t < strlist.count(); t += 2)
    {
	strlist[t].replace("<", "&lt;");
	strlist[t].replace(">", "&gt;");
    }
    msg = strlist.join("");
    msg.stripWhiteSpace();
    
    msg.replace('\n', "<br>");
}

QString MainForm::fullAutoAnswer()
{
    if (settings->getNowPlaying() && !nowplaying().isEmpty())
	return autoanswer.isEmpty() ? nowplaying() : autoanswer + ". " + nowplaying();
    else return autoanswer;
}

void MainForm::showMessage(const QString& from, const QString& text, bool mass)
{
    if (status == STATUS_OFFLINE) return;
    
    QString msg = text;

    if ((status != STATUS_DND) && (status != STATUS_AWAY) && settings->isSoundEnabled(Sound::Message))
	Sound::play(Sound::Message);
    
    msgparse(msg);
    
    if (settings->getShowPopups())
   {
	ReceiveMessageDlg *msgwin = new ReceiveMessageDlg();
	connect(msgwin, SIGNAL(reply(const QString &, const QString &)), this, SLOT(openSendMessage(const QString &, const QString &)));
	connect(msgwin, SIGNAL(chat(const QString &)), this, SLOT(openPrivate(const QString &)));
	connect(msgwin, SIGNAL(quote(const QString &, const QString &)), this, SLOT(openSendMessage(const QString &, const QString &)));
	connect(msgwin, SIGNAL(getinfo(const QString &)), this, SLOT(requestInfo(const QString &)));
	connect(msgwin, SIGNAL(openhistory(const QString &)), this, SLOT(openHistory(const QString &)));
	msgwin->setup(from, msg, text, mass);
	if (settings->getPrivShowAction() == 1) msgwin->showMinimized();
	else 
	{
	    msgwin->show();
	    if (settings->getPrivShowAction() == 2) msgwin->raise();
	}
    }
    
    showOnChannel(msg, from);
}

void MainForm::on_xxx_message(const QHostAddress &source, const QCString &sig, const QString &from, const QString &to, const QString &text)
{
    xxx->xxx_messageack(source, sig, status, from, nick, gender, fullAutoAnswer());
    showMessage(from, text, false);
}

void MainForm::on_xxx_massmessage(const QHostAddress &source, const QCString &sig, const QString &from, const QString &to, const QString &text)
{
    xxx->xxx_messageack(source, sig, status, from, nick, gender, fullAutoAnswer());
    if (settings->getAllowMass()) showMessage(from, text, MASS_MSG);
    if (settings->getShowMassOnChannel())
    {
	QString msg = text;
	
	msgparse(msg);
	
	showOnChannel(msg, from);
    }
}

void MainForm::showOnChannel(const QString &text, const QString &from)
{
    tmp = tr("%1Message from <a href=\"#N/m %2\">%3</a> received:<br>%4")
	  .arg(timestring(), urlenc(from), qtparse(from), text);
    QString chn = MAIN_CHANNEL;
    
    switch(settings->getPrivMsgAction())
    {
    case DONT_SHOW:
	tmp = tr("%1Message from <a href=\"#N/m %2\">%3</a> received")
	      .arg(timestring(), urlenc(from), qtparse(from));
	break;
    case SHOW_IN_MAIN:
	break;
    case SHOW_IN_CURRENT:
	chn = chattab->current();
	break;
    default: break;
    }
    chattab->write(chn, tmp, settings->getPrivateColor());
    log->write(datestring(), timestring(), from, MSG_TYPE, from, text);
    traynotify();
}

void MainForm::tcpPvtMsgReceived(const QString& from, const QString& text)
{
    showMessage(from, text, false);
}

void MainForm::on_xxx_hereack(const QString &to, const QString &channel, const QString &from, char ra)
{
    tmp = tr("%1Here <a href=\"#N%2\">%3</a> (%4)")
	  .arg(timestring(), urlenc(from), qtparse(from), ractivity[ra - RA_ACTIVE]);
    chattab->write(channel, tmp, settings->getNotifyColor());
}

void MainForm::on_xxx_messageack(char stat, const QString &to, const QString &from, char gend, const QString &answer)
{
    if ((to == nick) && (settings->getSystemMsgShow() & MSG_ACK_SHOW)) 
    {
	QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("received")) : QString(tr("received "));
	tmp = tr("%1%2 %3 your message in %4 mode")
	      .arg(timestring(), qtparse(from), gender_depend, status_lut[stat - STATUS_NORMAL]);
	if (!answer.isEmpty()) 
	{
	    userslist->setAutoanswer(from, answer);
	    tmp.append(QString("<br>%1").arg(answer));
	}
	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
    }
}

void MainForm::on_xxx_messageack(const QCString &sig, char stat, const QString &to, const QString &from, char gend, const QString &answer)
{
    on_xxx_messageack(stat, to, from, gend, answer); //TODO Make usable 'sig'
}

void MainForm::on_xxx_herereq(const QHostAddress &source, const QString &from, const QString &channel)
{
    if (chattab->exists(channel))
	xxx->xxx_hereack(source, from, channel, nick, (chattab->current() == channel) ? RA_SPEAKING : RA_CONNECTED);
    if (chattab->isPrivate(from) && channel == nick)
	xxx->xxx_hereack(source, from, channel, nick, (chattab->current() == from) ? RA_SPEAKING : RA_CONNECTED);
}

void MainForm::on_xxx_inforeq(const QHostAddress &source, const QString &to, const QString &from)
{
    if (settings->getSystemMsgShow() & INFO_REQ_SHOW)
    {
	QString gender_depend = ((int)userslist->getGender(from) == GENDER_MALE) ? QString(tr("requested")) : QString(tr("requested "));
    
	tmp = tr("%1 <a href=\"#N%2\">%3</a> %4 information about you")
	  .arg(timestring(), urlenc(from), qtparse(from), gender_depend);

	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
	traynotify();
    }
    
    if (!settings->getInfoACK())
    {
	if (settings->getSystemMsgShow() & INFO_REQ_SHOW) 
	    chattab->write(MAIN_CHANNEL, tr("Request refused"), settings->getNotifyColor());
	return;
    }
    
    QString software = (settings->getClientName().isEmpty()) ? QString(XXX_SOFTWARE) + QString(" v") + QString(VERSION) : settings->getClientName();
    
    xxx->xxx_inforeqack(source, from, nick, settings->getHostname(), settings->getLogonname(), settings->getIP() + ':' + QString::number(settings->getPort()), settings->getSendChList() ? chattab->channelsList() : "", fullAutoAnswer(), settings->getOS(), software, settings->getDomain(), settings->getFullName(), settings->getOccupation(), settings->getDepartment(), settings->getPhone(), settings->getMobile(), settings->getHomepage(), settings->getEmail(), settings->getAddress());
}

void MainForm::on_xxx_inforeqack(const QHostAddress &source, const QString &to, const QString &from, const QString &hostname, const QString &name, const QString &ip, const QString &channels, const QString &answer, const QString &os, const QString &domain, const QString &software, bool utf, const QString &fullname, const QString &occupation, const QString &department, const QString &phone, const QString &mobile, const QString &homepage, const QString &email, const QString &address)
{
    QString photofile;
    QString decautoanswer(answer);
    QUrl::decode(decautoanswer);
    
    if (to != nick) return;
    if (settings->getSystemMsgShow() & INFO_ACK_SHOW)
    {
	tmp = tr("%1 Information about %2:<br>Name: %3<br>Host: %4<br>")
	      .arg(timestring(), qtparse(from), qtparse(name), qtparse(hostname));
	tmp.append(tr("IP: %1<br>Platform: %2<br>Software: %3<br>Workgroup: %4<br>")
		   .arg(source.toString(), os, software, domain));
	tmp.append(tr("Codepage: %1<br>Channels: %2")
		   .arg(utf ? "Unicode" : "ANSI", qtparse(channels)));
	chattab->write(MAIN_CHANNEL, tmp, settings->getSystemColor());
    
	if (!answer.isEmpty())
	{
	    userslist->setAutoanswer(from, decautoanswer);
	    chattab->write(MAIN_CHANNEL, decautoanswer, settings->getSystemColor());
	}
    }
    
    QStringList chlist = QStringList::split("#", channels);
    availchannels->add(chlist);
    
    if (userslist->AvatarIsAPhoto(from))
    {
	QString uid;
	uid = xxx->getUuid(source);
	photofile =  QString(QDir::homeDirPath() + "/.trix/avatars/photo_" + uid);
    }
    else
	photofile = getUserAvatar(from);
    
    UserInfoDlg *udlg = findInfoDlg(xxx->getUuid(source));
    if (!udlg) udlg = addInfoDlg(xxx->getUuid(source));
    
    QString iface(settings->getIFbyIP(settings->getIP()));
    if (!iface.isNull()) 
	iface.truncate(iface.find(" ("));
    
    udlg->setup(from, hostname, name, os, utf, xxx->getProtoVersion(source), software, domain, channels, source.toString(), ((int)userslist->getGender(from) == GENDER_MALE) ? tr("Male") : tr("Female"), xxx->getHWAddr(source.toString(), iface), decautoanswer, fullname, occupation, department, phone, mobile, homepage, email, address, photofile, false);
    
    udlg->show();
}

void MainForm::photoRefresh(const QString &name)
{
    QString path = QString(QDir::homeDirPath() + "/.trix/avatars/photo_" + xxx->getUuid(userslist->getIP(name)));
    if (userslist->AvatarIsAPhoto(name)) xxx->xxx_httpGet(userslist->getIP(name).toString(), "/photo", path);
}

void MainForm::on_xxx_changeactive(const QString &from, char ractive)
{
    userslist->setRactive(from, ractive);
}

void MainForm::on_xxx_reqchannels(const QHostAddress& addr, const QString &from)
{
    if (settings->getSendChList()) xxx->xxx_reqchannelsack(addr, from, chattab->channelsList());
}

void MainForm::on_xxx_reqchannelsack(const QHostAddress& addr, const QString &from, const QString& channels)
{
    QStringList chlist = QStringList::split("#", channels);
    availchannels->add(chlist);
}

void MainForm::on_xxx_nickchange(const QString &from, const QString &to, char gend)
{
    QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("known")) : QString(tr("known "));
    tmp = tr("%1%2 is %3 as ")
	  .arg(timestring(), qtparse(from), gender_depend);
    tmp.append(tr("<a href=\"#N%1\">%2</a> from now")
	       .arg(urlenc(to), qtparse(to)));    
    if (settings->getSystemMsgShow() & NICK_CHANGE_SHOW)
    {
	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
	traynotify();
    }
    
    if (chattab->exists(from)) 
    {
	if (chattab->renamePrivate(from, to))
	{
	    if (settings->getSystemMsgShow() & NICK_CHANGE_SHOW)
		chattab->write(to, tmp, settings->getNotifyColor());
	}
    }
    userslist->newNick(from, to);
}

void MainForm::on_xxx_avatarchange(const QString &from, const QString &fname, char gend)
{
    if (settings->getSystemMsgShow() & NICK_CHANGE_SHOW)
    {
	QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("changes")) : QString(tr("changes "));
	tmp = tr("%1%2 %3 avatar")
	      .arg(timestring(), qtparse(from), gender_depend);
	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
	traynotify();
    }
    
    userslist->setGender(from, gend);
    userslist->setAvatar(from, fname);
    checkAvatar(userslist->getIP(from), fname);
}

void MainForm::on_xxx_avatarrequest(const QHostAddress &addr, const QString &fname)
{
    QString fpath = QDir::homeDirPath() + "/.trix/avatars/" + fname;
    std::cout << "Request for " << fpath << " received from " << addr.toString() << std::endl;
    
    if (QFile::exists(fpath))
	xxx->xxx_tcpfilesend(QStringList(fpath), "", getNick(addr), addr, true);
}

void MainForm::on_xxx_beep(const QHostAddress &source, const QString &to, const QString &from)
{
    if ((status != STATUS_NORMAL) || (to != nick)) return;
    
   tmp = tr("%1 Sound signal from %2 received")
	 .arg(timestring(), qtparse(from));
    chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
    
    xxx->xxx_beepack(source, from, nick, gender);
    
    if (settings->isSoundEnabled(Sound::Beep)) Sound::play(Sound::Beep);
    
    traynotify();    
}

void MainForm::on_xxx_beepack(const QString &to, const QString &from, char gend)
{
    if (to == nick)
    {
	QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("received")) : QString(tr("received "));
	tmp = tr("%1%2 %3 your signal")
	      .arg(timestring(), qtparse(from), gender_depend);
	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
    }
}

void MainForm::on_xxx_changestatus(const QString &uid, const QString &who, char stat, char gend, const QString &answer)
{
    if ( (userslist->setStatus(who, stat)) && (settings->getSystemMsgShow() & STATUS_SHOW) )
    {
	QString gender_depend = ((int)gend == GENDER_MALE) ? QString(tr("enters")) : QString(tr("enters "));

	tmp = QString("%1<a href=\"#N%2\">%3</a> %4")
	      .arg(timestring(), urlenc(who), qtparse(who), gender_depend);
	tmp.append(tr(" %1 mode")
		   .arg(status_lut[stat - STATUS_NORMAL]));
	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
	if (!answer.isEmpty())
	{
	    userslist->setAutoanswer(who, answer);
	    chattab->write(MAIN_CHANNEL, answer, settings->getNotifyColor());
	}
    }
    ChatDlg *c = findChatDlg(uid);
    if (c) c->setMode(status_lut[stat - STATUS_NORMAL]);
}

void MainForm::on_xxx_currenttopic(const QHostAddress &source, const QString &to, const QString &channel, const QString &topic)
{
    QString from = getNick(source);
    QString text = topic;
    if (from.isEmpty()) from = source.toString();
    if (to == nick && chattab->getTopic(channel).isEmpty() && !topic.isEmpty())
    {
	msgparse(text);
	tmp = tr("%1 Current topic on %2: %3 (Received from %4)")
	      .arg(timestring(), channel, text, qtparse(from));
	chattab->setTopic(channel, topic);
	chattab->write(channel, tmp, settings->getSystemColor());
	updateCaptionSlot();
    }
}

void MainForm::on_xxx_newtopic(const QHostAddress &source, const QString &channel, const QString &topic)
{
    QString from = getNick(source);
    QString text = topic;
    if (topic.isEmpty()) 
    {
	QString gender_depend = ((int)userslist->getGender(from) == GENDER_MALE) ? QString(tr("clears")) : QString(tr("clears "));
		
	chattab->clearTopic(channel);
	
	tmp = tr("%1 %2 %3 topic")
	      .arg(timestring(), qtparse(from), gender_depend);
    }
    else 
    {
	if (chattab->setTopic(channel, topic))
	{
	    msgparse(text);
	    QString gender_depend = ((int)userslist->getGender(from) == GENDER_MALE) ? QString(tr("changes")) : QString(tr("changes "));
	    tmp = tr("%1 %2 %3 topic to:<br>%4")
		  .arg(timestring(), qtparse(from), gender_depend, text);
	}
    }
    if (settings->getSystemMsgShow() & TOPIC_CHANGES_SHOW) 
    {
	chattab->write(channel, tmp, settings->getSystemColor());
	traynotify();
    }
    updateCaptionSlot();
}

void MainForm::on_xxx_ping(const QHostAddress &source, const QString &to, const QString &from, const QString &time, const QCString &bids)
{
    if (to == nick)
    {
	QTime current = QTime::currentTime();
	xxx->xxx_pong(source, from, nick, current.toString("hh:mm:ss"));
	addBoardMsg(source, bids);
    }
}

void MainForm::addBoardMsg(const QHostAddress &source, const QString &bids)
{
    QString reqstr;
    if (!bids.isEmpty() && settings->isBoardEnabled())
    {
	reqstr = board->addBoardMsgID(xxx->getUuid(source), bids, MSG_ABSENT);
	if (!reqstr.isEmpty()) xxx->xxx_boardmsgrequest(source, nick, reqstr);
    }    
}

void MainForm::on_xxx_pong(const QHostAddress &source, const QString &to, const QString &from, const QString &time, const QCString &bids)
{
    addBoardMsg(source, bids);
}

void MainForm::on_xxx_floodnotify(const QString &to, const QString &from, const QString &seconds)
{
    if (nick == to)
    {
	QString gender_depend = ((int)userslist->getGender(from) == GENDER_MALE) ? QString(tr("blocked")) : QString(tr("blocked "));
	tmp = tr("%1 %2 is %3 you for flooding by %4 seconds")
	      .arg(timestring(), qtparse(from), gender_depend, seconds);
    chattab->write(MAIN_CHANNEL, tmp, settings->getSystemColor());
    }
}

void MainForm::on_xxx_whohereneeded(const QHostAddress &source)
{
    xxx->xxx_whohere(source, nick);
}

void MainForm::on_userslistPingRequest(const QString &name)
{
    QTime current = QTime::currentTime();
    xxx->xxx_ping(userslist->getIP(name), name, nick, current.toString("hh:mm:ss"));
}

void MainForm::on_userslistWhohereRequest(const QString &name)
{
    xxx->xxx_whohere(userslist->getIP(name), nick);
}

void MainForm::on_userslistDeleteRequest(const QString &name)
{
    if (settings->getSystemMsgShow() & JOIN_LEAVE_SHOW) 
    {
	QString gender_depend = ((int)userslist->getGender(name) == GENDER_MALE) ? QString(tr("leaves")) : QString(tr("leaves "));
	tmp = tr("%1%2 %3 channel %4 (Timeout limit exceed)")
	      .arg(timestring(), qtparse(name), gender_depend, MAIN_CHANNEL);
   
	chattab->write(MAIN_CHANNEL, tmp, settings->getJoinLeaveColor());
	
	if (chattab->exists(name))
	{
	    if (chattab->isPrivate(name))
	    {
		tmp = tr("%1%2 %3 private chat (Timeout limit exceed)")
		      .arg(timestring(), qtparse(name), gender_depend);
		chattab->write(name, tmp, settings->getJoinLeaveColor());
	    }
	}
    }
     
    xxx->delUser(userslist->getIP(name));
    userslist->delUser(name);
    updateServiceMenu();
    
    if ((status == STATUS_NORMAL) && (chattab->current() == MAIN_CHANNEL) && settings->isSoundEnabled(Sound::LeaveChannel)) Sound::play(Sound::LeaveChannel);
}

bool MainForm::cmdparse(const QString &line)
{
    QString str = line;
    str = str.right(str.length() - 1);
    QString cmd = str.section(' ', 0, 0);

    CmdList::iterator it;
    for (it = cmdlist.begin(); it != cmdlist.end(); it++)
	if ((*it).command() == cmd)
	{
	   QString execline;
	   QString tmp;
	   QString channel = chattab->current();
	   QString arg[2];
	   if ((*it).argnum() == 1)
		arg[0] = str.section(' ', 1);
	    else
		if ((*it).argnum() == 2)
		{
		    arg[0] = str.section(' ', 1, 1);
		    arg[1] = str.section(' ', 2);
		}
		switch ((*it).id())
		{
		case CMD_ME:
			if (chattab->isPrivate())
			{
				xxx->xxx_private_me(userslist->getIP(channel), nick, channel, arg[0]);
				on_xxx_me(channel, nick, arg[0]);
			}
			else
				xxx->xxx_me(channel, nick, arg[0]);
			break;
		case CMD_HERE:
		    xxx->xxx_herereq(nick, channel);
		    break;
		case CMD_INFO:
		    requestInfo(arg[0]);
		    break;
		case CMD_BEEP:
		    beep(arg[0]);
		    break;
		case CMD_MESSAGE:
			if (str.contains(':')) 
			{
				tmp = str.right(str.length() - str.find(' ') - 1);
				arg[0] = tmp.section(':', 0, 0);
				arg[1] = tmp.right(tmp.length() - arg[0].length() - 1);
			}
			if (userslist->exists(arg[0]) && (!arg[1].isEmpty())) 
			{
				xxx->xxx_message(userslist->getIP(arg[0]), nick, arg[0], arg[1]);
				log->write(datestring(), timestring(), arg[0], MSG_TYPE, nick, arg[1]);
			}
			break;
		case CMD_MASSMESSAGE:
		    if (!arg[0].isEmpty())
		    {
			QStringList nicks = userslist->getAllUsers();
			nicks.remove(nick);
			sendMessage(nicks, arg[0]);
		    }
		    break;
		case CMD_NICK:
		    if (!arg[0].isNull())
			changeNick(arg[0]);
		    break;
		case CMD_TOPIC:
		    if (!arg[0].isNull())
			newTopic(channel, arg[0]);
		    else
		    {
			QString text = chattab->getTopic(channel);
			msgparse(text);
			tmp = tr("%1 Current topic on channel %2:<br>%3")
			      .arg(timestring(), qtparse(channel), text);
			chattab->write(channel, tmp, settings->getSystemColor());
		    }
		    break;
		case CMD_ADD:
		    if (arg[0].isEmpty()) 
			newTopic(channel, chattab->getTopic(channel));
		    else
			newTopic(channel, chattab->getTopic(channel) + " " + arg[0]);
		    break;
		case CMD_JOIN: 
		    chatline->clear();
		    joinChannel(arg[0]);
		    break;
		case CMD_LEAVE:
		    leaveChannel(channel);
		    break;
		case CMD_CHAT:
		   chatline->clear();
		   if (userslist->exists(arg[0]))
			  openPrivate(arg[0]);
		   break;
		case CMD_NORMAL:
		   goNORMAL();
		   break;
		case CMD_DND:
		   goDND();
		   break;
		case CMD_AWAY:
		   goAWAY();
		   break;
		case CMD_OFFLINE:
		   goOFFLINE();
		   break;
		case CMD_HELP:
		    {
			chattab->write(channel, tr("Available commands:"), settings->getSystemColor());
			CmdList::iterator ith;
			for (ith = cmdlist.begin(); ith != cmdlist.end(); ith++)
			    chattab->write(channel, " /" + QString((*ith).command()) + " - " + (*ith).descr(), settings->getSystemColor());
			break;
		    }
		 case CMD_EXEC:
			if (executing) return false;
			if (arg[0] == "-o")
			{
			   execline = arg[1];
			   onlylocal = false;
			}
			else
			{
			   execline = str.section(' ', 1);
			   onlylocal = true;
			}
			
			proc->setArguments(QStringList::split(' ', execline));
			if (proc->start())
			{
			   executing = true;
			   QTimer::singleShot(5000, this, SLOT(procTerminate()));
			}
			break;
		case CMD_QUIT:
		    closeSlot();
		break;
		case CMD_TEST:
		    //on_userslistWhohereRequest(arg[0]);
		break;
		default:  break;
		}
	return true;
    }
    return false;
}

void MainForm::procTerminate()
{
   proc->kill();
   executing = false;
}

void MainForm::procDataReady()
{
    QString res = proc->readStdout();
    if (onlylocal)
    {
	res.replace('\n', "<br>");

	tmp = QString("%1&lt;<a href=\"#N%2\">%3</a>&gt;: %4")
	      .arg(timestring(), urlenc(nick), qtparse(nick), QString::fromLocal8Bit(res));
	//TODO add parser for res variable
	chattab->write(chattab->current(), tmp, settings->getOwnColor());
    }
    else
    {
	chatline->setText(QString::fromLocal8Bit(res));
	sendLine();
    }
    procTerminate();
}

ChatDlg *MainForm::findChatDlg(const QString &uid)
{
    if (!cdmap.contains(uid))
	return NULL;
    return cdmap[uid];
}

UserInfoDlg *MainForm::findInfoDlg(const QString &uid)
{
    if (!infomap.contains(uid))
	return NULL;
    return infomap[uid];
}

ChatDlg *MainForm::addChatDlg(const QString &unick, const QString &uid)
{
    if (cdmap.contains(uid))
	return NULL;
    ChatDlg *c = new ChatDlg(0, "ChatDlgWidget", Qt::WDestructiveClose);
    connect(c, SIGNAL(closeChat(const QString &)), this, SLOT(closeChatDlg(const QString &)));
    connect(c, SIGNAL(getInfo(const QString &)), this, SLOT(requestInfo(const QString &)));
    connect(c, SIGNAL(sendFile(const QString &)), this, SLOT(filesendSlot(const QString &)));
    connect(c, SIGNAL(openHistory(const QString &)), this, SLOT(openHistory(const QString &)));
    connect(c, SIGNAL(sendMessage(const QString &, const QString &, const QString &)), this, SLOT(sendChatMessage(const QString &, const QString &, const QString &)));
    connect(c, SIGNAL(sendTypingMode(const QString&, uint)), xxx, SLOT(xxx_sendTypingMode(const QString &, uint)));
    c->setup(unick, uid, status_lut[userslist->getStatus(unick) - STATUS_NORMAL]);
    if (settings->getPrivShowAction() == 1)
	c->showMinimized();
    else
	c->show();
    if (xxx->xxx_createChatSession(userslist->getIP(unick), uid))
	c->setStatus(tr("Setting connection..."));
    else
	c->setReady();
    return cdmap[uid] = c;
}

void MainForm::sendChatMessage(const QString &uid, const QString &unick, const QString &text)
{
    QString msg = text;
    msgparse(msg);
    ChatDlg *c = (ChatDlg*) sender();
    xxx->xxx_sendChatMessage(uid, text);
    log->write(datestring(), timestring(), unick, PRV_TYPE, nick, msg);
    c->writeText(QString("<font color=\"#0000FF\">%1%2:</font><br>%3").arg(timestring(), nick, msg));
}

UserInfoDlg *MainForm::addInfoDlg(const QString &uid)
{
    if (infomap.contains(uid))
	return NULL;
    UserInfoDlg *u = new UserInfoDlg(0, "User info");
    connect(u, SIGNAL(closeInfo(const QString &)), this, SLOT(closeInfoDlg(const QString &)));
    connect(u, SIGNAL(channelClicked(const QString &)), this, SLOT(joinChannel(const QString &)));
    connect(u, SIGNAL(linkClicked(const QString &, const QString &)), this, SLOT(on_linkClicked(const QString &, const QString &)));
    connect(u, SIGNAL(refreshPhoto(const QString &)), this, SLOT(photoRefresh(const QString &)));
    connect(u, SIGNAL(changeAvatar()), this, SLOT(openAvatarsPage()));
    connect(u, SIGNAL(savePersonalInfo(const QString&, const QString&, const QString&, const QString&, const QString&, const QString&, const QString&, const QString&)), this, SLOT(savePersonal(const QString&, const QString&, const QString&, const QString&, const QString&, const QString&, const QString&, const QString&)));
    u->setUid(uid);
    return infomap[uid] = u;
}

void MainForm::closeInfoDlg(const QString &uid)
{
    delInfoDlg(uid);
}

void MainForm::closeChatDlg(const QString &uid)
{
    delChatDlg(uid);
}

bool MainForm::delChatDlg(const QString &uid)
{
    ChatDlg *c = findChatDlg(uid);
    if (c)
    {
	xxx->xxx_sleepChatSession(uid);
	cdmap.remove(uid);
	return true;
    }
    return false;
}

bool MainForm::delInfoDlg(const QString &uid)
{
    UserInfoDlg *u = findInfoDlg(uid);
    if (u)
    {
	infomap.remove(uid);
	delete u;
	return true;
    }
    return false;
}

void MainForm::openPrivateChatDlg(const QString &unick)
{
    ChatDlg *c;
    QString uid = xxx->getUuid(userslist->getIP(unick));
    c = findChatDlg(uid); 
    if (!c)
    {
	addChatDlg(unick, uid);
    }
    else
    {
	c->setWindowState(Qt::WindowActive);
	c->raise();
    }    
}

void MainForm::onUsersListDoubleClicked(const QString &unick)
{
    if ((xxx->getProtoVersionMajor(userslist->getIP(unick)) == 2) && (XXX_PROTOMAJOR == 2))
    {
	if (unick == nick) return;
	QString uid = xxx->getUuid(userslist->getIP(unick));
	if (xxx->xxx_isSessionEstablished(uid))
	{
	    openPrivateChatDlg(unick);
	    return;
	}
    }
    openSendMessage(unick, "");
}

void MainForm::openSendMessage( const QString &to, const QString &text)
{
    if (!to.isEmpty())
    {
	SendMessageDlg *msgwin = new SendMessageDlg(0, "Send message", Qt::WDestructiveClose);
	connect(msgwin, SIGNAL(send(const QStringList &, const QString &)), this, SLOT(sendMessage(const QStringList &, const QString &)));
	msgwin->setup(to, userslist->getAllUsers(), text, false, NULL);
	msgwin->show();
    }
}

void MainForm::openSendMassMessage()
{
    SendMessageDlg *msgwin = new SendMessageDlg(0, "Send mass message", Qt::WDestructiveClose);
    connect(msgwin, SIGNAL(send(const QStringList&, const QString&)), this, SLOT(sendMessage(const QStringList&, const QString&)));

    msgwin->setup(nick, userslist->getAllUsers(), "", true, NULL);

    msgwin->show();
}

void MainForm::openSendMassMessageSelected(const QStringList &nicks)
{
    SendMessageDlg *msgwin = new SendMessageDlg(0, "Send message", Qt::WDestructiveClose);    
    connect(msgwin, SIGNAL(send(const QStringList&, const QString&)), this, SLOT(sendMessage(const QStringList&, const QString&)));

    msgwin->setup("", userslist->getAllUsers(), "", false, nicks);

    msgwin->show();
}

void MainForm::openHistory(const QString &from)
{
    HistoryForm *hf = new HistoryForm(this, "History", TRUE);
    hf->setup(settings->getHistoryFilePath());
    hf->request(from);
    hf->show();
}

void MainForm::sendMessage(const QStringList &nicks, const QString &msg)
{
    QString str = msg;
    QStringList nl = nicks;
    if (!nl.empty()) 
    {
	if (nl.count() == 1) 
	{
	    xxx->xxx_message(userslist->getIP(nl[0]), nick, nl[0], str);
	    log->write(datestring(), timestring(), nl[0], MSG_TYPE, nick, str);
	}
	else 
	{
	    for (QStringList::Iterator it = nl.begin(); it!=nl.end(); ++it)
	    {
		User *user;
		if (user = userslist->getUser(*it))
		{
//		    xxx->xxx_message(user->getIP(), nick, *it, str);
		    xxx->xxx_massmessage(user->getIP(), nick, *it, str);
		}
		log->write(datestring(), timestring(), user->getNick(), MSG_TYPE, nick, str);
	    }
	}
    }
}

void MainForm::forceRefresh()
{
    userslist->clear();
    xxx->xxx_whohere(nick);
}

void MainForm::requestInfo(const QString &who)
{
    xxx->xxx_inforeq(userslist->getIP(who), who, nick);
}

void MainForm::filesendSlot(const QString& to)
{
    FileSendDlg *fs = new FileSendDlg(0, "File sending",Qt::WDestructiveClose|Qt::WStyle_Dialog);
    fs->show();
    fs->setup(to);
    connect( fs, SIGNAL(sendFiles(const QString&, const QStringList&, const QString&)), this, SLOT(filesSend(const QString&, const QStringList&, const QString&)) );
}

void MainForm::filesSend(const QString& receiver, const QStringList &filesnames, const QString& descr)
{
    if (!filesnames.isEmpty())
	xxx->xxx_tcpfilesend(filesnames, descr, receiver, userslist->getIP(receiver), false);
}

void MainForm::findByIPSlot()
{
    FindForm *ff = new FindForm(0, "Find by IP", Qt::WDestructiveClose|Qt::WStyle_Dialog);

    connect(ff, SIGNAL(find(const QString &)), this, SLOT(findHost(const QString &)));

    ff->show();
}

void MainForm::findHost(const QString &ip)
{
    ResultForm *rf = new ResultForm(0, "Search results", Qt::WDestructiveClose|Qt::WStyle_Dialog);

    rf->setup(userslist->getNicks(ip));

    connect(rf, SIGNAL(send(const QStringList &)), this, SLOT(openSendMassMessageSelected(const QStringList &)));

    rf->show();
}

void MainForm::updateCaptionSlot()
{
    if (!chattab->currentPage()->isBoard())
    {
	QString cap = QString("TriX [") + chattab->current() + QString(": ") + chattab->getTopic(chattab->current()) + QString("]");
	setCaption(cap);
    }
}

void MainForm::boardActivated()
{
    //TODO add short statistic
    setCaption(tr("TriX - Bulletin board"));
    chatChange_topicAction->setDisabled(true);
    editClear_chatAction->setDisabled(true);
    chatline->hide();
}

void MainForm::boardDisactivated()
{
    chatChange_topicAction->setEnabled(true);
    editClear_chatAction->setEnabled(true);
    chatline->show();
    QTimer::singleShot( 300, chattab, SLOT(scrollDownCurrent()) );
}

void MainForm::saveChatLine(const QString &chn)
{
    chattab->saveChatLine(chn, chatline->text());
}

void MainForm::loadChatLine(const QString &chn)
{
    chatline->setText(chattab->loadChatLine(chn));
    chatline->moveCursor(QTextEdit::MoveEnd, false);
}

void MainForm::usersListUpdate()
{
    if (settings->getLegacyStyle()) forceRefresh();
    else xxx->xxx_whohere(nick);
}

void MainForm::changeNick(const QString &newnick)
{
    QString n = newnick;
    n.replace(QChar(':'), "_" );
    if (!userslist->exists(n))
    {
	xxx->xxx_nickchange(nick, n, gender);
	nick = n;
	if (nicksmap.contains(nick))
	{
	    QString tmp = nicksmap[nick];
	    gender = QChar(tmp[0]);
	    userpixmap = tmp.right(tmp.length() - 1);
	    changeAvatar(userpixmap);
	}
	else
	{
	    nicksmap.insert(nick, QString(QChar(gender) + userpixmap));
	    updateNicksMenu();
	}
	nickedit->setText(nick);
	userslist->setOwnNick(nick);
    }
}

void MainForm::changeAvatar(const QString& fname)
{
    xxx->xxx_avatarchange(nick, fname, gender);
}

void MainForm::imhereToAll()
{
    QStringList nl = userslist->getAllUsers();
    for (QStringList::Iterator it = nl.begin(); it!=nl.end(); ++it)
    {
	User *user;
	if (user = userslist->getUser(*it))
	    xxx->xxx_imhere(userslist->getIP(*it), *it, nick, status, gender, winstate, userpixmap, settings->getOwnColor());
    }
}

void MainForm::joinChannel(const QString &channel)
{
    if (channel.isEmpty())
	return;
    
    const QString name = (channel[0]=='#') ? channel : '#' + channel;
	
    if (!chattab->exists(name))
    {
	chattab->addChannel(name, settings->getBackgroundColor(), settings->getChatFont());
	xxx->xxx_join(nick, name, status, gender, userpixmap, settings->getOwnColor());
    }
}

void MainForm::leaveChannel(const QString &channel)
{
    if (chattab->isPrivate(channel))
    {
	chattab->delPrivate(channel);
	xxx->xxx_private_leave(userslist->getIP(channel), nick, channel, gender);
    }
    else
	if ((channel.lower() != QString(MAIN_CHANNEL).lower()))
	{
	    chattab->delChannel(channel);
	    xxx->xxx_leave(nick, channel, gender);
	}
}

void MainForm::goNORMAL()
{
    xxx->xxx_changestatus(nick, status = STATUS_NORMAL, gender, autoanswer=QString(""));
    trayicon = Icons::icon_common;
    tray->setIcon(trayicon.pixmap());
}

void MainForm::goDND()
{
    StatusForm *sf = new StatusForm(this, tr("Enter DND message"), settings->getDNDAnswer());
    connect (sf, SIGNAL(changeStatus(const QString &)), this, SLOT(DNDStatus(const QString &)));
    sf->show();
}

void MainForm::DNDStatus(const QString &answer)
{
    xxx->xxx_changestatus(nick, status = STATUS_DND, gender, autoanswer = answer);
    settings->setDNDAnswer(answer);
    trayicon = Icons::icon_user_dnd;
    tray->setIcon(trayicon.pixmap());
}

void MainForm::goAWAY()
{
    StatusForm *sf = new StatusForm(this, tr("Enter Away message"), settings->getAwayAnswer());
    connect (sf, SIGNAL(changeStatus(const QString &)), this, SLOT(AwayStatus(const QString &)));
    sf->show();
}

void MainForm::AwayStatus(const QString &answer)
{
    xxx->xxx_changestatus(nick, status = STATUS_AWAY, gender, autoanswer = answer);
    settings->setAwayAnswer(answer);
    trayicon = Icons::icon_user_away;
    tray->setIcon(trayicon.pixmap());
}

void MainForm::goOFFLINE()
{
    StatusForm *sf = new StatusForm(this, tr("Enter Offline message"), settings->getOfflineAnswer());
    connect (sf, SIGNAL(changeStatus(const QString &)), this, SLOT(OfflineStatus(const QString &)));
    sf->show();
}

void MainForm::OfflineStatus(const QString &answer)
{
    xxx->xxx_changestatus(nick, status = STATUS_OFFLINE, gender, autoanswer = answer);
    settings->setOfflineAnswer(answer);
    trayicon = Icons::icon_user_offline;
    tray->setIcon(trayicon.pixmap());
}

void MainForm::onChannelDoubleClicked(const QString &chn, int para, int pos)
{
    changeTopic();
}

void MainForm::changeTopic()
{
    TopicForm *tf = new TopicForm(this, tr("Enter new topic for channel "), chattab->current(), chattab->getTopic(chattab->current()));
    connect (tf, SIGNAL(change(const QString &, const QString &)), this, SLOT(newTopic(const QString &, const QString &)));
    tf->show();
}

void MainForm::on_editPersonal()
{
    QString software = (settings->getClientName().isEmpty()) ? QString(XXX_SOFTWARE) + QString(" v") + QString(VERSION) : settings->getClientName();
    QString photofile = getLocalAvatar();
    UserInfoDlg *udlg = findInfoDlg(xxx->getLocalUid());
    if (!udlg) udlg = addInfoDlg(xxx->getLocalUid());

    QString iface(settings->getIFbyIP(settings->getIP()));
    iface.truncate(iface.find(" ("));

    udlg->setup(nick, settings->getHostname(), settings->getLogonname(), settings->getOS(), settings->getUseUTF(), xxx->getProtoVersion(settings->getIP()), software, settings->getDomain(), chattab->channelsList(), settings->getIP(), ((int)settings->getGender() == GENDER_MALE) ? tr("Male") : tr("Female"), xxx->getHWAddr(settings->getIP(), iface), fullAutoAnswer(), settings->getFullName(), settings->getOccupation(), settings->getDepartment(), settings->getPhone(), settings->getMobile(), settings->getHomepage(), settings->getEmail(), settings->getAddress(), photofile, true);
    
    udlg->show();
}

void MainForm::savePersonal(const QString& fn, const QString& occ, const QString& dep, const QString& ph, const QString& mb, const QString& hp, const QString& em, const QString& addr)
{
    settings->setFullName(fn);
    settings->setOccupation(occ);
    settings->setDepartment(dep);
    settings->setPhone(ph);
    settings->setMobile(mb);
    settings->setHomepage(hp);
    settings->setEmail(em);
    settings->setAddress(addr);
}

void MainForm::on_clearTopic(const QString &channel)
{
    xxx->xxx_newtopic(channel, "");
}

void MainForm::newTopic(const QString &channel, const QString &txt)
{
    QString text;
    QString changer = " (" + nick + ")";
    if (txt.right(changer.length()) == changer) text = txt;
    else text = txt + changer;
    xxx->xxx_newtopic(channel, text);
}

void MainForm::openPrivate(const QString &who)
{
   if (who.isEmpty())
	return;
   
   if ((xxx->getProtoVersionMajor(userslist->getIP(who)) == 2) && (XXX_PROTOMAJOR == 2))
       openPrivateChatDlg(who);
   else
   {
       if (!chattab->exists(who))
       {
	   chattab->addPrivate(who, settings->getBackgroundColor(), settings->getChatFont());
	   xxx->xxx_private_join(userslist->getIP(who), nick, who, gender);
/*       
       if (xxx->getProtoVersionMajor(userslist->getIP(who)) == 2)
       {
	   tmp = tr("%1Remote side does not support old version of the private chat.").arg(timestring());
	   chattab->write(who, tmp, settings->getNotifyColor());
       }
*/
       }
   }
}

void MainForm::reconfigureNetwork()
{
    xxx->closeSockets();

    xxx->setupInterface(settings->getIP(), settings->getBCastAddress(), settings->getMulticastGroup(), settings->getTTL(), settings->getPort(), settings->getUseMulticast());
    xxx->setUseUTF(settings->getUseUTF());
    xxx->setAnsiCodec((settings->getEncoding() != "") ? QTextCodec::codecForName(settings->getEncoding()) : QTextCodec::codecForLocale());
    joinChannel(MAIN_CHANNEL);
    if (!settings->getShowRA()) xxx->xxx_changeactive(nick, winstate = WIN_NOTACTIVE);

    QTimer::singleShot( 10000, this, SLOT(usersListUpdate()) );
}

void MainForm::LogActionSlot()
{
    HistoryForm *hf = new HistoryForm(this, "History", TRUE);
    hf->setup(settings->getHistoryFilePath());
    hf->show();
}

void MainForm::nickedit_returnPressed()
{
    changeNick(nickedit->text());
}

void MainForm::nickbutton_clicked()
{
    nicksmenu->popup(QCursor::pos());
}

void MainForm::toggleMultiLine()
{
    chatline->toggleMultiLine();
    QTimer::singleShot( 300, chattab, SLOT(scrollDownCurrent()) );
}

void MainForm::mouseMoveEvent ( QMouseEvent * e )
{
    resetAwayTimer();
}

void MainForm::resetActivityTimer(const QHostAddress &addr)
{
    userslist->resetActivityTimer(addr);
}

void MainForm::resetAwayTimer()
{
    awaytimer = 0;
     if ((autoaway == true) && (status == STATUS_AWAY))
    {
	autoaway = false;
	goNORMAL();
    }
}

void MainForm::tcpFileRecvFinished(const QString& from, const QString& fname)
{
    if (settings->getSystemMsgShow() & FILE_MSG_SHOW)
    {
	tmp = tr("%1File \"%2\" was received from %3")
	      .arg(timestring(), fname, qtparse(from));
	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
    }
}

void MainForm::httpFileRecvFinished(const QString& uid, const QString& fname)
{
    if (fname == "/photo")
    {
	UserInfoDlg *u = findInfoDlg(uid);
	if (u)
	    u->checkPhoto();
    }
    if (uid == "BOARD_IMAGE")
    {
	bimgtimer->start(3000, true);
    }
}

void MainForm::tcpAvatarRecvFinished(const QString& from, const QString& fname)
{
    userslist->setAvatar(from , fname);
}

void MainForm::tcpFileSendFinished(const QString& to, const QString& fname)
{
    if (settings->getSystemMsgShow() & FILE_MSG_SHOW) 
    {
	tmp = tr("%1File \"%2\" was sended to %3")
	      .arg(timestring(), fname, qtparse(to));
	chattab->write(MAIN_CHANNEL, tmp, settings->getNotifyColor());
    }
}

void MainForm::tcpRecvChatMessage(const QHostAddress &addr, const QString &text)
{
    QString uid = xxx->getUuid(addr);
    QString unick = getNick(addr);
    QString msg = text;
    
    ChatDlg *c = findChatDlg(uid);
    if (c == NULL) 
    {
	c = addChatDlg(unick, uid);
	if (c) c->setReady();
    }
    
    msgparse(msg);
    
    if (c) 
    {
	log->write(datestring(), timestring(), unick, PRV_TYPE, unick, msg);
	c->writeText(QString("<font color=\"#FF0000\">%1%2:</font><br>%3").arg(timestring(), unick, msg));
	if (settings->getPrivShowAction() == 2) c->raise();
	if (!c->isActiveWindow()) c->notify(true);
    }
}

void MainForm::tcpChatSessionEstablished(const QString &uid)
{
    ChatDlg *c = findChatDlg(uid);
    if (c) c->setEstablished();
}

void MainForm::tcpCloseChatSession(const QString &uid)
{
    ChatDlg *c = findChatDlg(uid);
    if (c) c->setClosed();
}

void MainForm::tcpRecvTypingMode(const QString &uid, uint tmode)
{
    ChatDlg *c = findChatDlg(uid);
    if (c) 
    {
	switch(tmode)
	{
	case 0:
	    c->setStatus(tr("Typing..."));
	    break;
	    
	case 1:
	    c->setStatus(tr("Ready"));
	    break;
	    
	default: break;
	}
    }
}

QString MainForm::getParam(const QString &sep, const QString &bigstr, uint &pos)
{
    QString str;
    int i = bigstr.find(sep, pos);
    str = bigstr.mid(pos, i  - pos);
    pos =  i + sep.length();  
    return str;
}

QString MainForm::parseBoard(const QString &bigstr)
{
    QString str;
    QString size, author, tpc, text, uuid, pdate, ptime, m1, edate, etime, m2, port, ip, sign, id;
    QStringList biglist = QStringList::split(SEPARATOR, bigstr);
    
    QRegExp r("<\\s*A\\s*href\\s*=\\s*\"http:", false);
    r.setMinimal(true);

    for (uint i = 0; i < biglist.count(); i++)
    {
	uint pos = 0;
	size = getParam("<SIZE>", biglist[i], pos);
	author = board->decode(getParam("<AUTHOR>", biglist[i], pos));
	tpc = board->decode(getParam("<TOPIC>", biglist[i], pos));
	text = board->decode(getParam("<TXT>", biglist[i], pos));
	uuid = getParam("<UUID>", biglist[i], pos);
	pdate = getParam("<PDATE>", biglist[i], pos);
	ptime = getParam("<PTIME>", biglist[i], pos);
	m1 = getParam("<MAGIC1>", biglist[i], pos);
	edate = getParam("<EDATE>", biglist[i], pos);
	etime = getParam("<ETIME>", biglist[i], pos);
	m2 = getParam("<MAGIC2>", biglist[i], pos);
	port = getParam("<PORT>", biglist[i], pos);
	ip = getParam("<IP>", biglist[i], pos);
	sign = getParam("<SIGN>", biglist[i], pos);
	id = getParam("<ID>", biglist[i], pos);
	
	text.replace(r, "<A href = \"#Lhttp%3A");

	imgParse(text);
	
	if (!bimap.contains(id))
	    bimap[id] = NORMAL_SHOWN;

	str.append(tr("<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"left\" style=\"color:#777777;font-size:12pt;font-weight:bold;\">%1</td><td align=\"right\"><a href=\"#h%2\"><img src=\"../gfx/%3\"></a></td></tr></table>").arg(tpc, id, (bimap[id] == NORMAL_SHOWN || bimap[id] == OWN_SHOWN) ? "uparrows.png" : "downarrows.png"));
	
	if (bimap[id] == NORMAL_SHOWN || bimap[id] == OWN_SHOWN)
	{
	    str.append(tr("<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" style=\"font-size:8pt;\"><tr><td style=\"color:#777777;font-weight:bold;\">Author: <a href=\"#A%1\" style=\"color:#3333cc;font-weight:bold;\">%2</a></td><td align=\"right\" style=\"font-style:italic;\">Date: %3</td></tr></table>").arg(uuid, author, pdate));
	    str.append(tr("<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\"><tr><td align=\"left\">%1</td></tr></table><table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" style=\"color:#111111;font-size:8pt;\"><tr><td align=\"right\"><a href=\"#R%2\"><img src=\"../gfx/trash.png\"> Remove</a></td>").arg(text, id));
	    
	    if (bimap[id] == OWN_SHOWN)
		str.append(tr("<td align=\"right\"><a href=\"#E%1\"><img src=\"../gfx/p4img2.jpg\">Edit</a></td>").arg(id, id));
	    
	    str.append(tr("</tr></table>"));
	}
	str.append(tr("<hr color=\"#000000\"><br>"));
    }

    return str;
}

void MainForm::updateBoard()
{
    QString str;
   
    chattab->clearBoard();
    
    str = QString(tr("<table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#336799\"><tr><td style=\"font-size:18pt;font-weight:bold;font-style:italic;font-family:Sans-Serif;color:#ffffff;\"><nobr>Bulletin board</nobr></td></tr></table><table width=\"100%\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#4991C3\"><tr><td style=\"font-size:8pt;font-family:Sans-Serif;color:#ffffff;\">%1 bulletin(s)</td><td align=\"right\" style=\"font-size:8pt;font-family:Sans-Serif;color:#ffffff;\"><a href=\"#O\"><img src=\"../gfx/settings.png\">Options</a></td></tr></table><table width=\"100%\" style=\"font-size:8pt;font-family:Sans-Serif;\" cellspacing=\"0\" cellpadding=\"0\" bgcolor=\"#A0C9E7\"><tr><td align=\"right\" style=\"font-size:8pt;font-family:Sans-Serif;color:#111111;\"><a href=\"#N\"><img src=\"../gfx/new.png\">New</a>    <a href=\"#H\"><img src=\"../gfx/uparrows.png\"><nobr>Hide all</nobr></a>    <a href=\"#S\"><img src=\"../gfx/downarrows.png\"><nobr>Show all</nobr></a>    <a href=\"#M\"><img src=\"../gfx/%2\"><nobr>Hide/show my</nobr></a></td></tr></table>").arg(QString::number(board->actMessages().count()), hideownbmsgs ? "cbchecked.png" : "cbblank.png"));
    
    chattab->writeToBoard(str);
    
    str = parseBoard(board->getAll());
    
    if (!bimglock)
    {
	bimglock = true;
	QTimer::singleShot(10000, this, SLOT(releaseBoardImgLock()));
    }
    
    chattab->writeToBoard(str);
}

void MainForm::imgParse(QString &str)
{
    DWORD dwHashLen;
    CRYPTHASH *pCryptHash;
    unsigned char abHashValue[RSAENH_MAX_HASH_SIZE];
    QString imgstr;
    QStringList list;
    QFileInfo fi;
    
    QRegExp rx("<.*img.*src.*=.*\"(.*)\".*>", false);
    
    QDir d(QDir::homeDirPath() + "/.trix/cache");
    
    int pos = 0;
    
    if (!d.exists()) 
	d.mkdir(d.absPath());
    
    QString basepath = d.absPath();
    
    rx.setMinimal(true);
    
    while (pos >= 0)
    {
	pos = rx.search(str, pos);
	if (pos > -1) 
	{
	    if (!list.contains(rx.cap(1))) 
		list += rx.cap(1);
	    pos += rx.matchedLength();
	}
    }
    
    for (uint i = 0; i < list.count(); i++)
    {
	CryptCreateHash(CALG_MD5, 0, &pCryptHash);
	if (pCryptHash)
	{
	    if (CryptHashData(pCryptHash, (const unsigned char*)list[i].local8Bit().data(), list[i].local8Bit().size(), 0)) 
	    {
		dwHashLen = RSAENH_MAX_HASH_SIZE;
		if (CryptGetHashParam(pCryptHash, HP_HASHVAL, (unsigned char*)abHashValue, &dwHashLen, 0))
		{
		    imgstr = QString("img_") + Conversions::raw2str(abHashValue, dwHashLen);
		    QString savepath = basepath +"/" + imgstr;
		    QUrl url(list[i]);
		    fi.setFile(savepath);
		    if (!fi.exists())
		    {
			if (url.hasHost() && url.hasPath() && xxx)
			{
			    if (!bimglock)
			    {
				xxx->xxx_httpGet(url.host(), (url.hasPort()) ? url.port() : 80, "BOARD_IMAGE", url.path(), savepath);
			    }
			}
		    }
		}
	    }
	    CryptDestroyHash(pCryptHash);
	}
	str.replace(list[i], imgstr);
    }
}

void MainForm::releaseBoardImgLock()
{
    bimglock = false;
}

QString MainForm::getNick(const QHostAddress&addr)
{
    return userslist->getNick(addr);
}

QString MainForm::getDownloadPath()
{
    QString dp;
    dp = settings->getDownloadPath();
    if (settings->getCreateSenderFolder()) dp.prepend('1');
    else dp.prepend('0');
    return dp;
}

bool MainForm::getAcceptWOPrompt()
{
    return settings->getAcceptWOPrompt();
}

int MainForm::getMaxAvatarSize()
{
    return settings->getMaxAvatarSize();
}

char MainForm::getLocalStatus()
{
    return status;
}

char MainForm::getLocalGender()
{
    return gender;
}

const QString& MainForm::getLocalNick()
{
    return nick;
}

QString MainForm::getLocalAnswer()
{
    return fullAutoAnswer();
}

QString MainForm::getLocalFile() //TODO add filename
{
    QString fl;
    fl = "";
    return fl;
}

QString MainForm::getUserAvatar(const QString& name)
{
    return getAvatarFile(userslist->getAvatar(name));
}

QString MainForm::getAvatarFile(const QString &a)
{
    QString av;
    if (!a.isEmpty()) {
	av = QString(QDir::homeDirPath() + "/.trix/avatars/" + a);
	if (QFile::exists(av)) return av;
	av = QString(QString(DATADIR) + "avatars/" + a);
	if (QFile::exists(av)) return av;
	av = QString(QString(DATADIR) + "gfx/" + a);
	if (QFile::exists(av)) return av;
    }
    return av;
}

QString MainForm::getLocalAvatar()
{
    return getAvatarFile(userpixmap);
}

void MainForm::openChannelsList()
{
    availchannels->show();
}

void MainForm::refreshChList()
{
    xxx->xxx_reqchannels(nick);
}

void MainForm::editClear_chatAction_activated()
{
    chattab->clearCurrent();
}

void MainForm::editClear_lineAction_activated()
{
    chatline->clear();
}

void MainForm::boardMsgUpdated(const QHostAddress &source, const QString &uid, Q_UINT32 id)
{
    if (settings->isBoardEnabled())
	xxx->xxx_boardmsgrequest(source, nick, QString::number(id, 36));
}

void MainForm::boardMsgDeleted(const QHostAddress &source, const QString &uid, Q_UINT32 id)
{
    if (settings->isBoardEnabled())
    {
	board->delBoardMsg(id);
	updateBoard();
    }
}

void MainForm::tcpMessageACKReceived(const QString& from, const QString& answer, char stat)
{
    on_xxx_messageack(stat, nick, from, userslist->getGender(from), answer);
}

void MainForm::updateServiceMenu()
{
    if(smlocked)
    {
	if (!smtimer->isActive()) smtimer->start(2000, true);
	return;
    }
    
    QFile f;
    QFileInfo fi(servicemenupath);
    QDir d(fi.dir(true));
    
    smlocked = true;
    
    if (!d.exists()) 
	d.mkdir(d.absPath());
    
    QStringList nicks = userslist->getAllUsers();
    QString users;
    QString av;
    
    for (uint i = 0; i < nicks.count(); i++)
	users.append("user" + QString::number(i) + ";");

    f.setName(servicemenupath);
    f.open(IO_ReadWrite);
    
    QTextStream s(&f);
    s.setEncoding(QTextStream::UnicodeUTF8);
    
    s << "[Desktop Entry]" << "\n";
    s << "Actions=" << users << "\n";
    s << "ServiceTypes=all/allfiles" << "\n";
    s << tr("X-KDE-Submenu=TriX - Send files") << "\n";

    for (uint i = 0; i < nicks.count(); i++)
    {
	s << QString("[Desktop Action %1]").arg("user" + QString::number(i)) << "\n";
	s << QString("Exec=trix -s user=%1 files=%U").arg(xxx->getUuid(userslist->getIP(nicks[i]))) << "\n";
	av = getUserAvatar(nicks[i]);
	av.replace("DEFAULT_MALE", "male.png");
	av.replace("DEFAULT_FEMALE", "female.png");
	
	s << "Icon=" << av << "\n";
	s << "Name=" << nicks[i] << "\n";
    }
    f.close();
    
    smlocked = false;
}

void MainForm::on_xxx_sendtoprocess(const QHostAddress &source, const QString &uid, const QString &fn)
{
    if (uid.startsWith("user=") && fn.startsWith("files="))
    {
	QString unick = getNick(xxx->getIP(uid.right(uid.length() - 5)));
	FileSendDlg *fs = new FileSendDlg(0, "File sending",Qt::WDestructiveClose|Qt::WStyle_Dialog);
	fs->setup(unick, fn.right(fn.length() - 6));
	connect( fs, SIGNAL(sendFiles(const QString&, const QStringList&, const QString&)), this, SLOT(filesSend(const QString&, const QStringList&, const QString&)) );
	fs->show();
	fs->raise();
    }
}

void MainForm::setLocalAutoanswerInToolTip()
{
    userslist->setAutoanswer(nick, fullAutoAnswer());
}

void MainForm::addToBoard(BoardMsg *bmsg)
{
    if (settings->isBoardEnabled())
    {
	board->addBoardMsg(bmsg);
	btimer->start(3000, true);
    }
}

void MainForm::releaseBoardLock()
{
    brlock = false;
}

void MainForm::requestMissed(const QHostAddress &source)
{
    if (settings->isBoardEnabled())
    {
	if (!board->missed().isEmpty())
	{
	    if (!brlock) {
		if (!requested.contains(xxx->getUuid(source)))
		{
		    brlock = true;
		    QTimer::singleShot(5000, this, SLOT(releaseBoardLock()));
		    requested.append(xxx->getUuid(source));
		    if (settings->requestFromAuthorsOnly())
		    {
			QString bids = board->missed(xxx->getUuid(source)).join(",");
			xxx->xxx_boardmsgrequest(source, nick, bids);
		    }
		    else
			xxx->xxx_requestmissed(source, nick, board->missed().join(","));
		}
	    }
	}
    }
}
