#ifndef __SETTINGS_H
#define __SETTINGS_H

#include "sound.h"
#include "crypto.h"
#include <quuid.h>
#include <qstring.h>
#include <qcolor.h>
#include <qfont.h>
#include <qsettings.h>
#include <qmap.h>

#define JOIN_LEAVE_SHOW		1
#define STATUS_SHOW		2
#define NICK_CHANGE_SHOW	4
#define FILE_MSG_SHOW		8
#define INFO_REQ_SHOW		16
#define MSG_ACK_SHOW		32
#define INFO_ACK_SHOW		64
#define TOPIC_CHANGES_SHOW	128

class Settings
{
private:
    QString def_npfile, def_nick, def_ip, def_topic, def_playcmd, def_local, def_bcastaddress, def_mcastgrp;
    QString def_back, def_own, def_users, def_system, def_notify, def_prv, def_joinleave, def_userslistbackcolor, def_userslisttextcolor, def_nickeditbackcolor, def_nickedittextcolor, def_chatlineeditbackcolor, def_chatlineedittextcolor; //default colors
    QString def_dndanswer, def_awayanswer, def_offlineanswer; //default answers
    QString def_webbrowser, def_smbbrowser, def_emailclient, def_ftpclient;
    QString def_histpath, def_downloadpath;
    QString def_encoding;
    QString def_snd[Sound::Dummy];
    Sound::SoundType def_sound;
	
    uint def_port, def_ttl;
    int def_maxhistsize, def_maxavatarsize, def_maxlinescount;
    int def_awaylimit;
    int def_autoupdateinterval;
    
    QUuid uuid;
    QString npfile, nick, logonname, hostname, ip, topic, playcmd, local, bcastaddress, mcastgrp, os, useros, clientname, domain;
    QColor backgroundcolor, privatecolor, joinleavecolor, notifycolor, systemcolor, userscolor, owncolor, userslistbackcolor, userslisttextcolor, nickeditbackcolor, nickedittextcolor, chatlineeditbackcolor, chatlineedittextcolor;
    QFont chatfont, userslistfont, editlinefont;
    QString dndanswer, awayanswer, offlineanswer; 
    QString webbrowser, smbbrowser, emailclient, ftpclient;
    QString histpath, downloadpath;
    QString snd[Sound::Dummy];
    QString fullname, occupation, department, phone, mobile, homepage, email, address;
    Sound::SoundType sound;     
    bool sndp[Sound::Dummy];  //play-don't play flags

    uint port, ttl;
    int privmsgaction, privshowaction;
    int maxhistsize, maxavatarsize, maxlinescount;
    int awaylimit;
    int autoupdateinterval;

    bool showmassonchannel, allowmass, infoack, showra, startchannels, sendchlist, showpopups, useamarok;
    bool usemcast, useutf, keep_status, np, startmin, acceptwoprompt, multiline, legacystyle;
    bool secondsintimestamp;
    bool useuserscolors;
    bool boardenabled;
    bool requestfromauthorsonly;
    bool autoupdate;
    bool createsenderfolder;
    bool systraynotify;
	
    QString userpixmap; //avatar filename
    QString channelslist; //channels list separated by ','
    QString lastnicks; //nicks list separated by ','
	
    unsigned char gender;
    char systemmsgshow;
    int x, y, width, height, framewidth;
    int refresh, status;
    QSettings cfg;
    QString version, encoding;
    typedef QMap<QString, QString> IFMap;
    IFMap ifname;
	
    CRYPTKEY *pKey;

    void setUUID(const QUuid &u) { uuid = u; cfg.writeEntry("/General/UUID", uuid.toString()); }
    void getInterfaces();
     
    const QString GetSystemVersion();
    const QString GetEnv(char *);

    void setVersion(const QString &v) 
    { 
	cfg.writeEntry("/General/Version", this->version = v); 
    }
     
    const QString getSampleKey(Sound::Sfx s)
    {
	QString key;
	switch (s)
	{
	case Sound::Chatline: key = "/Chatline"; break;
	case Sound::Message: key = "/Message"; break;
	case Sound::JoinChannel: key = "/Join"; break;
	case Sound::LeaveChannel: key = "/Leave"; break;
	case Sound::Beep: key = "/Beep"; break;
	default: break;
	}
	return key;
    }

public:
    Settings();
    virtual ~Settings();

    void getDefaults();
    bool load();
	
    //
    // Get methods
    bool getKeepStatus()   { return keep_status; }
    bool getStartMin() { return startmin; }
    bool getShowRA() { return showra; }
    bool getUseUTF()   { return useutf; }
    bool getAllowMass() { return allowmass; }
    bool getShowMassOnChannel() { return showmassonchannel; }
    bool getNowPlaying()   { return np; }
    bool getInfoACK() { return infoack; }
    bool getSendChList() { return sendchlist; }
    bool getSampleEnabled(Sound::Sfx i) { return sndp[i]; }
    bool getStartChannels() { return startchannels; }
    bool isSoundEnabled(Sound::Sfx i) { return sndp[i]; }
    bool getAcceptWOPrompt() { return acceptwoprompt; }
    bool getMultiline() { return multiline; }
    bool getUseMulticast() { return usemcast; }
    bool getShowPopups() { return showpopups; }
    bool getUseAmaroK() { return useamarok; }
    bool getLegacyStyle() { return legacystyle; }
    bool getTimeStampWithSeconds() { return secondsintimestamp; }
    bool getUseUsersColors() { return useuserscolors; }
    bool isBoardEnabled() { return boardenabled; }
    bool requestFromAuthorsOnly() { return requestfromauthorsonly; }
    bool isSystrayNotifyEnabled() { return systraynotify; }
    bool getAutoUpdate() { return autoupdate; }
    bool getCreateSenderFolder() { return createsenderfolder; }

    unsigned char getGender()  { return gender; }
    char getSystemMsgShow() { return systemmsgshow; }

    const uint getPort()   { return port; }
    uint getDefPort() { return def_port; }
    uint getDefTTL() { return def_ttl; }
    uint getTTL() { return ttl; }
  
    int getPrivMsgAction() { return privmsgaction; }
    int getPrivShowAction() { return privshowaction; }

    int getStatus()    { return status; }
    int getFrameWidth()   { return framewidth; }
    int getWidth()    { return width; }
    int getHeight()    { return height; }
    int getX()    { return x; }
    int getY()    { return y; }

    int getMaxHistSize() { return maxhistsize; }
    int getMaxAvatarSize() { return maxavatarsize; }
    int getMaxLinesCount() { return maxlinescount; }
  
    int getAwayLimit() { return awaylimit; }
    int getAutoUpdateInterval() { return autoupdateinterval; }

    const QString& getTopic()  { return topic; }
    const QString& getNick()   { return nick; }
    const QString& getIP()   { return ip; }
    const QString& getLogonname()  { return logonname; }  
    const QString& getHostname()  { return hostname; }
    const QString& getOS()   { return (useros.isEmpty()) ? os : useros; }
    const QString& getUserOS()   { return useros; }
    const QString& getClientName()   { return clientname; }
    const QString& getDomain()   { return domain; }
    const QString& getBCastAddress() { return bcastaddress; }
    const QString& getMulticastGroup() { return mcastgrp; }
    const QString& getEncoding()  { return encoding; }
    const QString& getNPFile() {return npfile; }
    const QString& getVersion()  { return version; }
    const QString& getPlayCommand() { return playcmd; }
    const QString& getSampleFileName(Sound::Sfx i) { return snd[i]; }
    const QString& getChannelsList() { return channelslist; }
    const QString& getHistoryFilePath() { return histpath; }
    const QString& getDownloadPath() { return downloadpath; }
    const QString& getLastNicks() { return lastnicks; }
    const QString& getFullName() { return fullname; }
    const QString& getOccupation() { return occupation; }
    const QString& getDepartment() { return department; }
    const QString& getPhone() { return phone; }
    const QString& getMobile() { return mobile; }
    const QString& getHomepage() { return homepage; }
    const QString& getEmail() { return email; }
    const QString& getAddress() { return address; }
  
    const QString& getUserPixmap()   { return userpixmap; }
    const QString& getLocal()   { return local; }

    const QString& getDNDAnswer()   { return dndanswer; }
    const QString& getAwayAnswer()  { return awayanswer; }
    const QString& getOfflineAnswer()  { return offlineanswer; }

    const QString& getWebBrowser() { return webbrowser; }
    const QString& getSMBBrowser() { return smbbrowser; }
    const QString& getEmailClient() { return emailclient; }
    const QString& getFTPClient() { return ftpclient; }
  
    QString getIPbyIF(const QString& ifn) { return ifname.contains(ifn) ? ifname[ifn] : QString(NULL); }
    QString getIFbyIP(const QString& ifip);
    QStringList getIFNames();
  
    QColor getBackgroundColor()  { return backgroundcolor; }
    QColor getOwnColor()   { return owncolor; }
    QColor getUsersColor()   { return userscolor; }
    QColor getSystemColor()   { return systemcolor; }
    QColor getNotifyColor()   { return notifycolor; }
    QColor getJoinLeaveColor()  { return joinleavecolor; }
    QColor getPrivateColor()   { return privatecolor; }
    QColor getUsersListBackColor() { return userslistbackcolor; }
    QColor getUsersListTextColor() { return userslisttextcolor; }
    QColor getNickEditBackColor() { return nickeditbackcolor; }
    QColor getNickEditTextColor() { return nickedittextcolor; }
    QColor getChatLineEditBackColor() { return chatlineeditbackcolor; }
    QColor getChatLineEditTextColor() { return chatlineedittextcolor; }
  
    QFont getChatFont()   { return chatfont; }
    QFont getUsersListFont()   { return userslistfont; }
    QFont getEditLineFont()   { return editlinefont; }
  
    Sound::SoundType getSoundType() { return sound; }

    const QUuid& getUUID() const { return uuid; }
  
    CRYPTKEY * getPKey() { return pKey; }
  
    //
    // Set methods
    void setNick(const QString &nick) { cfg.writeEntry("/Network/Nick", this->nick = nick); }
//    void setHostname(const QString &hostname) { cfg.writeEntry("/Network/Hostname", this->hostname = hostname); }
    bool setIP(const QString &ip) { if (this->ip != ip) { cfg.writeEntry("/Network/IP", this->ip = ip); return true; } else return false; }      
    bool setPort(uint p) { if (port != p) { cfg.writeEntry("/Network/Port", static_cast<int>(port = p)); return true; } else return false; }
    bool setTTL(uint t) { if (ttl != t) { cfg.writeEntry("/Network/TTL", static_cast<int>(ttl=t)); return true; } else return false; }
    bool setBCastAddress(const QString &a) { if (bcastaddress != a) { cfg.writeEntry("/Network/BroadcastAddress", bcastaddress = a);  return true; } else return false; }
    bool setMulticast(const QString &ip) { if (mcastgrp != ip) { cfg.writeEntry("/Network/MulticastGroup", mcastgrp = ip);  return true; } else return false; }
    bool setUseMulticast(bool u) { if (usemcast != u) { cfg.writeEntry("/Network/UseMulticast", usemcast=u); return true; } else return false; }
    void setGeometry(int x, int y, int w, int h, int f)
    {
	cfg.beginGroup("/Window");
	cfg.writeEntry("/x", this->x = x);
	cfg.writeEntry("/y", this->y = y);
	cfg.writeEntry("/Width", this->width = w);
	cfg.writeEntry("/Height", this->height = h);
	cfg.writeEntry("/FrameWidth", this->framewidth = f);
	cfg.endGroup();
    }
    void setAutoanswers(QString dndanswer, QString awayanswer, QString offlineanswer)
    {
	cfg.beginGroup("/Autoanswers");
	cfg.writeEntry("/DND", this->dndanswer = dndanswer);
	cfg.writeEntry("/Away", this->awayanswer = awayanswer);
	cfg.writeEntry("/Offline", this->offlineanswer = offlineanswer);
	cfg.endGroup();
    }  
    void setDNDAnswer(QString a) { cfg.writeEntry("/Autoanswers/DND", dndanswer = a); }
    void setAwayAnswer(QString a) {  cfg.writeEntry("/Autoanswers/Away", awayanswer = a); }
    void setOfflineAnswer(QString a) {cfg.writeEntry("/Autoanswers/Offline", offlineanswer = a); }
  
    void setWebBrowser(QString w) { cfg.writeEntry("/Misc/WebBrowser", webbrowser = w); }
    void setSMBBrowser(QString s) { cfg.writeEntry("/Misc/SMBBrowser", smbbrowser = s); }
    void setEmailClient(QString e) { cfg.writeEntry("/Misc/EmailClient", emailclient = e); }
    void setFTPClient(QString f) { cfg.writeEntry("/Misc/FTPClient", ftpclient = f); }
  
    void setFonts(QFont chatfont, QFont userslistfont, QFont editlinefont)
  {
	this->chatfont = chatfont;
	this->userslistfont = userslistfont;
	this->editlinefont = editlinefont;
	cfg.beginGroup("/Fonts");
	cfg.writeEntry("/Chat", chatfont.toString());
	cfg.writeEntry("/UsersList", userslistfont.toString());
	cfg.writeEntry("/EditLine", editlinefont.toString());
	cfg.endGroup();
    }  
    void setColors(QColor back, QColor own, QColor users, QColor prv, QColor system, QColor notify, QColor joinleave)
    {
	this->backgroundcolor=back;
	this->owncolor=own;
	this->userscolor=users;
	this->privatecolor=prv;
	this->systemcolor=system;
	this->notifycolor=notify;
	this->joinleavecolor=joinleave;
      
	cfg.beginGroup("/Colors");
	cfg.writeEntry("/Background", back.name());
	cfg.writeEntry("/Own", own.name());
	cfg.writeEntry("/Users", users.name());
	cfg.writeEntry("/Private", prv.name());
	cfg.writeEntry("/System", system.name());
	cfg.writeEntry("/Notify", notify.name());
	cfg.writeEntry("/Join-Leave", joinleave.name());
	cfg.endGroup();
    }
    void setUsersListBackColor(QColor c) {userslistbackcolor=c; cfg.writeEntry("/Colors/UsersListBackColor", c.name()); }
    void setUsersListTextColor(QColor c) {userslisttextcolor=c; cfg.writeEntry("/Colors/UsersListTextColor", c.name()); }
    void setNickEditBackColor(QColor c) {nickeditbackcolor=c; cfg.writeEntry("/Colors/NickEditBackColor", c.name()); }
    void setNickEditTextColor(QColor c) {nickedittextcolor=c; cfg.writeEntry("/Colors/NickEditTextColor", c.name()); }
    void setChatLineEditBackColor(QColor c) {chatlineeditbackcolor=c; cfg.writeEntry("/Colors/ChatLineEditBackColor", c.name()); }
    void setChatLineEditTextColor(QColor c) {chatlineedittextcolor=c; cfg.writeEntry("/Colors/ChatLineEditTextColor", c.name()); }
	
    void setPrivMsgAction(int a) { cfg.writeEntry("/General/PrivMsgAction", privmsgaction=a); }
    void setPrivShowAction(int a) { cfg.writeEntry("/General/PrivShowAction", privshowaction=a); }
    void setMultiline(bool m) { cfg.writeEntry("/General/Multiline", multiline=m); }
    void setAcceptWOPrompt(bool a) { cfg.writeEntry("/General/AcceptWithoutPrompt", acceptwoprompt=a); }
    void setKeepStatus(bool f) { cfg.writeEntry("/General/KeepStatus", keep_status = f); }	  void setShowRA(bool r) { cfg.writeEntry("/General/ShowRemoteActive", showra = r); }
    void setNowPlaying(bool n) { cfg.writeEntry("/Misc/NowPlaying", np = n); }
    void setNPFile(const QString &f) { cfg.writeEntry("/Misc/NPFile", npfile = f); }
    void setStatus(int s) { cfg.writeEntry("/General/Status", status = s); }
    void setAwayLimit(int a) { cfg.writeEntry("/General/AwayLimit", awaylimit = a); }
    void setMaxHistSize(int s) { cfg.writeEntry("/General/MaxHistorySize", maxhistsize=s); }
    void setMaxAvatarSize(int s) { cfg.writeEntry("/General/MaxAvatarSize", maxavatarsize=s); }
    void setMaxLinesCount(int l) { cfg.writeEntry("/General/MaxLinesCount", maxlinescount=l); }
    void setTopic(const QString &t) { cfg.writeEntry("/General/Topic", topic = t); }
    void setUserOS(const QString &o) { cfg.writeEntry("/Misc/UserOS", useros = o); }
    void setClientName(const QString &n) { cfg.writeEntry("/Misc/ClientName", clientname = n); }
    bool setLocal(const QString &l) { if (local != l) { cfg.writeEntry("/General/Local", local = l); return true; } else return false; } 
    bool setEncodingName(const QString e) { if (encoding != e) { cfg.writeEntry("/Misc/Encoding", encoding = e); return true; } else return false; }
    bool setUseUTF(bool f) { if (useutf != f) {cfg.writeEntry("/Misc/UTF", useutf = f);  return true; } else return false; }
    
    void setShowPopups(bool p) { cfg.writeEntry("/General/ShowPopups", showpopups = p); }
    void setAllowMass(bool a) { cfg.writeEntry("/General/AllowMass", allowmass = a); }
    void setShowMassOnChannel(bool m) { cfg.writeEntry("/General/ShowMassOnChannel", showmassonchannel = m); }
    void setInfoACK(bool i) { cfg.writeEntry("/General/InfoACK", infoack = i); }
    void setStartChannels(bool s) { cfg.writeEntry("/Misc/OpenChannelsOnStart", startchannels = s); }
    void setUseAmaroK(bool u) { cfg.writeEntry("/Misc/UseAmaroK", useamarok = u); }
    void setSendChList(bool s) { cfg.writeEntry("/Misc/SendChannelsList", sendchlist = s); }
    void setGender(unsigned char g) { cfg.writeEntry("/General/Gender", gender = g); }
    void setSystemMsgShow(char m) { cfg.writeEntry("/General/SystemMessages", systemmsgshow = m); }
    void setStartMin(bool s) { cfg.writeEntry("/General/StartMinimized", startmin = s); }
    void setLegacyStyle(bool l) { cfg.writeEntry("/General/LegacyUpdateStyle", legacystyle = l); }
    void setTimeStampWithSeconds(bool t) { cfg.writeEntry("/Misc/TimeStampWithSeconds", secondsintimestamp = t); }
    void setUseUsersColors(bool u) { cfg.writeEntry("/Misc/UseUsersColors", useuserscolors = u); }
    void setBoardEnabled(bool b) { cfg.writeEntry("/Misc/BoardEnabled", boardenabled = b); }
    void setRequestFromAuthorsOnly(bool r) { cfg.writeEntry("/Misc/RequestFromAuthorsOnly", requestfromauthorsonly = r); }
    void setSystrayNotifyEnabled(bool s) { cfg.writeEntry("/Misc/SystrayNotifyEnabled", systraynotify = s); }
    void setCreateSenderFolder(bool c) { cfg.writeEntry("/General/CreateSenderFolder", createsenderfolder = c); }
    void setAutoUpdate(bool a) { cfg.writeEntry("/General/AutoUpdate", autoupdate = a); }
    void setAutoUpdateInterval(int i) { cfg.writeEntry("/General/AutoUpdateInterval", autoupdateinterval = i); }
    void setUserPixmap(const QString &fid) { cfg.writeEntry("/Misc/UserPixmap", userpixmap = fid); }
    void setSoundType(Sound::SoundType t) { cfg.writeEntry("/Sound/sound", sound = t); }
    void setSampleFileName(Sound::Sfx s, const QString &f)
    {
	QString key = getSampleKey(s);
	if (!key.isEmpty())
	{
	    cfg.beginGroup("/Wavs");
	    cfg.writeEntry(key, snd[s] = f);
	    cfg.endGroup();
	 }
    }
    void setSampleEnabled(Sound::Sfx s, bool f)
    {
	QString key = getSampleKey(s);
	if (!key.isEmpty())
	{
	    cfg.beginGroup("/WavFlags");
	    cfg.writeEntry(key, sndp[s] = f);
	    cfg.endGroup();
	}
    }
    void setPlayCommand(const QString &c) { cfg.writeEntry("/Sound/PlayCommand", playcmd = c); }
    void setChannelsList(const QString &l) { cfg.writeEntry("/Misc/ChannelsList", channelslist = l); }
    void setHistoryFilePath(const QString &p) { cfg.writeEntry("/General/HistoryFile", histpath=p); }
    void setDownloadPath(const QString &p) { cfg.writeEntry("/General/DownloadPath", downloadpath=p); }
    void setLastNicks(const QString &ln) { cfg.writeEntry("/General/LastNicks", lastnicks = ln); }
    void setFullName(const QString &f) { cfg.writeEntry("/Personal/FullName", fullname = f); }
    void setOccupation(const QString &o) { cfg.writeEntry("/Personal/Occupation", occupation = o); }
    void setDepartment(const QString &d) { cfg.writeEntry("/Personal/Department", department = d); }
    void setPhone(const QString &p) { cfg.writeEntry("/Personal/Phone", phone = p); }
    void setMobile(const QString &m) { cfg.writeEntry("/Personal/Mobile", mobile = m); }
    void setHomepage(const QString &h) { cfg.writeEntry("/Personal/Homepage", homepage = h); }
    void setEmail(const QString &e) { cfg.writeEntry("/Personal/Email", email = e); }
    void setAddress(const QString &a) { cfg.writeEntry("/Personal/Address", address = a);	 }
};

#endif
