#include "tcpserver.h"
#include "xxxprotocol.h"
//#include <iostream>

TCPServer::TCPServer( const QHostAddress& ip, Q_UINT16 port, QObject *parent , CRYPTKEY *pCryptKey) : QServerSocket( ip, port, 1, parent )
{
    if ( !ok() )
	qWarning("Failed to bind to port %d", port);
    pkey = pCryptKey;
}

void TCPServer::newConnection( int socket )
{
    TCPSocket* s = new TCPSocket( this, pkey );
    connect(s, SIGNAL(boardUpdated(const QByteArray &)), this, SIGNAL(tcpBoardUpdated(const QByteArray &)));
    connect(s, SIGNAL(fileRecvFinished(const QString&, const QString&)), this, SLOT(fileReceived(const QString&, const QString&)));
    connect(s, SIGNAL(chatSessionEstablishedByRemote(const QString&)), this, SLOT(addChatSession(const QString&)));
    connect(s, SIGNAL(pvtMsgReceived(const QHostAddress&, const QString&, const QString&, const QString&, const QString&, const QString&)), this, SIGNAL(tcpPvtMsgReceived(const QHostAddress&, const QString&, const QString&, const QString&, const QString&, const QString&)));
    connect(s, SIGNAL(messageACKReceived(const QHostAddress&, const QString&, const QString&, const QString&, char)), this, SIGNAL(tcpMessageACKReceived(const QHostAddress&, const QString&, const QString&, const QString&, char)));
    connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );
    
    s->setSocket( socket );
}

void TCPServer::sendFile(const QStringList& filesnames, const QString& descr, const QString& receiver, const QString& uuid, const QHostAddress& addr, Q_UINT16 port, bool avatar)
{
    TCPSocket* s = new TCPSocket( this, pkey );
    connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(fileSendFinished(const QString&, const QString&)), this, SLOT(fileSended(const QString&, const QString&)) );
    s->sendFile(filesnames, descr, receiver, uuid, addr, port, avatar);
}

void TCPServer::httpGet(const QString& addr, Q_UINT16 port, const QString& uid, const QString& frq, const QString& fsd)
{
    TCPSocket* s = new TCPSocket( this, pkey );
    connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(fileRecvFinished(const QString&, const QString&)), this, SLOT(httpFileReceived(const QString&, const QString&)) );
    s->httpGet(addr, port, uid, frq, fsd);
}

TCPSocket *TCPServer::findChatSession(const QString &uid)
{
    if (!smap.contains(uid))
	return NULL;
    return smap[uid];
}

TCPSocket *TCPServer::createChatSession(const QHostAddress& addr, Q_UINT16 port, const QString &uid, CRYPTKEY *pubkey)
{
    TCPSocket *s;
    
    if(!smap.contains(uid))
    {
	s = new TCPSocket( this, pkey );
	s->createChatSession(addr, port, uid, pubkey);
	smap[uid] = s;
	connect(s, SIGNAL(chatSessionEstablished(const QString&)), this, SIGNAL(tcpChatSessionEstablished(const QString&)));
	connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
	connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );
	connect( s, SIGNAL(recvChatMessage(const QHostAddress&, const QString&)), this, SIGNAL(tcpRecvChatMessage(const QHostAddress&, const QString&)));
	connect( s, SIGNAL(recvTypingMode(const QString&, uint)), this, SIGNAL(tcpRecvTypingMode(const QString&, uint)));
	return s;
    }
    
    s = findChatSession(uid);
    if (s) s->wakeupChatSession();
    
    return NULL;
}

void TCPServer::sleepChatSession(const QString &uid)
{
    TCPSocket* s = findChatSession(uid);
    if (s) 
	s->sleepChatSession();
}

void TCPServer::wakeupChatSession(const QString &uid)
{
    TCPSocket* s = findChatSession(uid);
    if (s) 
	s->wakeupChatSession();
}

void TCPServer::addChatSession(const QString &uid)
{
    TCPSocket *s = (TCPSocket*) sender();
    
    if(s && !smap.contains(uid))
    {
	smap[uid] = s;
	connect( s, SIGNAL(recvChatMessage(const QHostAddress&, const QString&)), this, SIGNAL(tcpRecvChatMessage(const QHostAddress&, const QString&)));
	connect( s, SIGNAL(recvTypingMode(const QString&, uint)), this, SIGNAL(tcpRecvTypingMode(const QString&, uint)));
	emit tcpChatSessionEstablished(uid);
    }
}

bool TCPServer::delChatSession(const QString &uid)
{
    TCPSocket *s = findChatSession(uid);
    if (s)
    {
	smap.remove(uid);
	emit tcpCloseChatSession(uid);
	delete s;
	return true;
    }
    return false;
}

void TCPServer::sendChatMessage(const QString &uid, const QString &text)
{
    TCPSocket* s = findChatSession(uid);
    if (s == NULL) 
    {
	//TODO reconnect
	return;
    }
    s->sendChatMessage(text);
}

void TCPServer::sendTypingMode(const QString &uid, uint tmode)
{
    TCPSocket* s = findChatSession(uid);
    if (s) s->sendTypingMode(tmode);
}

void TCPServer::boardMsgRequest(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &bids, Q_UINT16 port)
{
    TCPSocket* s = new TCPSocket( this, pkey );
    connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );

    s->boardMsgRequest(addr, nick, uid, bids, port);
}

void TCPServer::requestMissed(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &bids, Q_UINT16 port)
{
    TCPSocket* s = new TCPSocket( this, pkey );
    connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );

    s->requestMissed(addr, nick, uid, bids, port);
}

void TCPServer::sendMessageACK(const QHostAddress& addr, const QString& nick, const QString& uid, char status, const QString& answer, const QCString& sig, Q_UINT16 port)
{
    TCPSocket* s = new TCPSocket( this, pkey );
    connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );
    
    s->sendMessageACK(addr, nick, uid, status, answer, sig, port);
}

void TCPServer::sendMessage(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &text, Q_UINT16 port)
{
    TCPSocket* s = new TCPSocket( this, pkey );
    connect(s, SIGNAL(messageACKReceived(const QHostAddress&, const QString&, const QString&, const QString&, char)), this, SIGNAL(tcpMessageACKReceived(const QHostAddress&, const QString&, const QString&, const QString&, char)));
    connect( s, SIGNAL(closeConnection()), this, SLOT(closeSocket()) );
    connect( s, SIGNAL(connectionClosed()), this, SLOT(closeSocket()) );
    
    s->sendMessage(addr, nick, uid, text, port);
}

void TCPServer::fileReceived(const QString &sndr, const QString &fname)
{
    TCPSocket *socket = (TCPSocket*)sender();
    if (socket->messageType() == TCP_AVATAR_TRANSFER) emit tcpAvatarRecvFinished(sndr, fname);
    if (socket->messageType() == TCP_FILE_TRANSFER) emit tcpFileRecvFinished(sndr, fname);
}

void TCPServer::httpFileReceived(const QString&sndr, const QString& fname)
{
    emit httpFileRecvFinished(sndr, fname);
}

void TCPServer::fileSended(const QString&receiver, const QString& fname)
{
    emit tcpFileSendFinished(receiver, fname);
}

void TCPServer::closeSocket()
{
    TCPSocket *socket = (TCPSocket*)sender();
    
    if (socket->isChatSession())
	delChatSession(socket->getSessUid());
    else
	delete socket;
}

QString TCPServer::getNick(const QHostAddress& addr)
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getNick(addr);
}

QString TCPServer::getDownloadPath()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getDownloadPath();
}

bool TCPServer::getAcceptWOPrompt()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getAcceptWOPrompt();
}

int TCPServer::getMaxAvatarSize()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getMaxAvatarSize();
}

char TCPServer::getLocalGender()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getLocalGender();
}

char TCPServer::getLocalStatus()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getLocalStatus();
}

QString TCPServer::getLocalNick()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getLocalNick();
}

QString TCPServer::getLocalUid()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getLocalUid();
}

QString TCPServer::getLocalAnswer()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getLocalAnswer();
}

QString TCPServer::getLocalAvatar()
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getLocalAvatar();
}

QString TCPServer::getLocalFile() //TODO add filename
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getLocalFile();
}

QString TCPServer::getUuid(const QHostAddress& addr)
{
    XXXProtocol *par = (XXXProtocol*)parent();
    return par->getUuid(addr);
}
