#ifndef __TCPSERVER_H
#define __TCPSERVER_H

#include "tcpsocket.h"

#include <qserversocket.h>

class TCPServer : public QServerSocket
{
    Q_OBJECT

public:
    TCPServer( const QHostAddress& ip, Q_UINT16 port, QObject *parent, CRYPTKEY *pCryptKey);
    
    void newConnection( int socket );
    void httpGet(const QString& addr, Q_UINT16 port, const QString& uid, const QString& frq, const QString& fsd);
    void sendFile(const QStringList& filesnames, const QString& descr, const QString& receiver, const QString& uuid, const QHostAddress& addr,  Q_UINT16 port, bool avatar);
    void boardMsgRequest(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &bids, Q_UINT16 port);
    void requestMissed(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &bids, Q_UINT16 port);
    void sendMessageACK(const QHostAddress& addr, const QString& nick, const QString& uid, char status, const QString& answer, const QCString& sig, Q_UINT16 port);
    void sendMessage(const QHostAddress& addr, const QString& nick, const QString& uid, const QString &text, Q_UINT16 port);
    void sendChatMessage(const QString &uid, const QString &text);
    void sendTypingMode(const QString &uid, uint tmode);
    void  sleepChatSession(const QString &);
    void  wakeupChatSession(const QString &);

    TCPSocket *findChatSession(const QString&);
    TCPSocket *createChatSession(const QHostAddress &addr, Q_UINT16 port, const QString &uid, CRYPTKEY *pubkey);
    
    QString getLocalNick();
    QString getLocalAnswer();
    QString getLocalAvatar();
    QString getLocalFile();
    QString getLocalUid();
    QString getNick(const QHostAddress& addr);
    QString getUuid(const QHostAddress& addr);
    QString getDownloadPath();
    bool getAcceptWOPrompt();
    int getMaxAvatarSize();
    char getLocalGender();
    char getLocalStatus();
    

signals:
    void tcpFileRecvFinished(const QString&, const QString&);
    void tcpCloseChatSession(const QString&);
    void tcpChatSessionEstablished(const QString&);
    void tcpRecvChatMessage(const QHostAddress&, const QString&);
    void httpFileRecvFinished(const QString&, const QString&);
    void tcpAvatarRecvFinished(const QString&, const QString&);
    void tcpFileSendFinished(const QString&, const QString&);
    void tcpMessageACKReceived(const QHostAddress&, const QString&, const QString&, const QString&, char);
    void tcpPvtMsgReceived(const QHostAddress &, const QString &, const QString &, const QString &, const QString &, const QString &);
    void tcpRecvTypingMode(const QString&, uint);
    void tcpBoardUpdated(const QByteArray &);
    
private slots:
    void closeSocket();
    void httpFileReceived(const QString&, const QString&);
    void fileReceived(const QString&, const QString&);
    void fileSended(const QString&, const QString&);
    void addChatSession(const QString &);
    
private:
    CRYPTKEY *pkey;
    QMap <QString, TCPSocket *> smap;
    
    bool delChatSession(const QString&);
};

#endif
