/***************************************************************************
                          userinfodlg.ui.h  -  description
                             -------------------
    begin                : Thu Feb 16 19:41:00 EST 2005
    copyright            : (C) 2004-2007 by Sergey Litvinov
    email                : linch@adsl.by
 ***************************************************************************/

/***************************************************************************
 *                                                                         					   *
 *   This program is free software; you can redistribute it and/or modify  		   *
 *   it under the terms of the GNU General Public License as published by  		   *
 *   the Free Software Foundation; either version 2 of the License, or     		   *
 *   (at your option) any later version.                                   				   *
 *                                                                         					   *
 ***************************************************************************/

void UserInfoDlg::init()
{
    pf = NULL;
}

void UserInfoDlg::setup(const QString& from, const QString& hostname, const QString& name, const QString& os, bool utf, const QString &protoversion, const QString& software, const QString& domain, const QString& channels, const QString& ip, const QString& gender, const QString &hwaddr, const QString &answer, const QString &fullname, const QString &occupation, const QString &department, const QString &phone, const QString &mobile, const QString &homepage, const QString &email, const QString &address, const QString &photofile, bool e)
{
    editable = e;
    nick = from;
    photofilename = photofile;
    
    checkPhoto();
    
    QStringList tmpchnls;
    QString cp = utf ? "UNICODE" : "ANSI";
    setCaption(tr("Information on ") + from);
    HostnameLabel->setText(hostname);
    NameLabel->setText(name);
    OSLabel->setText(os);
    SoftwareLabel->setText(software);
    UserIPLabel->setText(ip);
    DomainLabel->setText(domain);
    MACAddressLabel->setText(hwaddr);
    AutoanswerLabel->setText(answer);
    ProtocolVersion->setText(protoversion + ", " + cp);
    FullNameLabel->setText(fullname);
    OccupationLabel->setText(occupation);
    DepartmentLabel->setText(department);
    PhoneLabel->setText(phone);
    MobileLabel->setText(mobile);
    WWWLabel->setText(homepage);
    EmailLabel->setText(email);
    AddressLabel->setText(address);
    
    if (!editable)
    {
	OKButton->hide();
	QToolTip::add( photolabel, tr("Left click - refresh photo, right click - show full size photo") );
    }  
    else
    {
	FullNameLabel->setReadOnly(false);
	OccupationLabel->setReadOnly(false);
	DepartmentLabel->setReadOnly(false);
	PhoneLabel->setReadOnly(false);
	MobileLabel->setReadOnly(false);
	PhoneLabel->setReadOnly(false);
	WWWLabel->setReadOnly(false);
	EmailLabel->setReadOnly(false);
	AddressLabel->setReadOnly(false);
	
	CloseButton->setCaption(tr("Cancel"));
	
	QToolTip::add( photolabel, tr("Click to change avatar") );
    }

    channelsList->clear();
    
    tmpchnls =  QStringList::split("#", channels);
    for (uint i = 0; i < tmpchnls.count(); i++)
	tmpchnls[i].prepend("#");
    channelsList->insertStringList(tmpchnls);
}

void UserInfoDlg::checkPhoto()
{
    if (QFile::exists(photofilename))
    {
	QImage photoimg;
	QPixmap photopix;
	photoimg.load(photofilename);
	photopix = photoimg.smoothScale(photoFrame->maximumWidth(), photoFrame->maximumHeight(), QImage::ScaleMin);
	
	photolabel->setPixmap(photopix);
    }
    else 
	emit refreshPhoto(nick);
}

void UserInfoDlg::channelsList_doubleClicked( QListBoxItem * it)
{
    emit channelClicked(it->text());
}


void UserInfoDlg::hostClicked(QMouseEvent *e)
{
    emit linkClicked(QString("#S\\\\" + HostnameLabel->text()), QString("Info on " + nick));
}

void UserInfoDlg::OKButton_clicked()
{
    emit savePersonalInfo(FullNameLabel->text(), OccupationLabel->text(), DepartmentLabel->text(), PhoneLabel->text(), MobileLabel->text(), WWWLabel->text(), EmailLabel->text(), AddressLabel->text());
    close();
}

void UserInfoDlg::photoLabelMouseEvent(QMouseEvent *e)
{
    if (e->button() == LeftButton)
    {
	if(!editable) emit refreshPhoto(nick);	
	else {
	    emit changeAvatar();
	}
    }
    if (e->button() == RightButton)
	if(!editable) showFullSize();
}

void UserInfoDlg::setUid(const QString& u)
{
    uid = u;
}

void UserInfoDlg::setPhoto(const QString& photoname)
{
    photofilename = photoname;
    checkPhoto();
}

void UserInfoDlg::showFullSize()
{
    QImage photoimg;
    photoimg.load(photofilename);
    if (photoimg.isNull()) return;
    if (photoimg.width() > qApp->desktop()->width() || photoimg.height() > qApp->desktop()->height()) photoimg = photoimg.smoothScale(qApp->desktop()->width(), qApp->desktop()->height(), QImage::ScaleMin);
    if (!pf) {
	pf = new PhotoForm(0, "Photo", Qt::WStyle_Customize | Qt::WStyle_Dialog | Qt::WStyle_NoBorder);
	connect(pf, SIGNAL(closePhoto()), this, SLOT(photoClose()));
    }
    pf->setImage(photoimg);
    pf->show();
}

void UserInfoDlg::photoClose()
{
    pf->hide();
}

void UserInfoDlg::closeEvent(QCloseEvent *event)
{
    emit closeInfo(uid);
    event->accept();
}

void UserInfoDlg::destroy()
{
    if (pf) delete pf;
}
